from  ...mechanics import *
from  ...mechanics import ReportComponent as BaseReportComponent

from . import pl

import pandas as pd
import numpy as np
from sympy import lambdify

from sympy import *
from pandas import *
from sympy.physics.mechanics import dynamicsymbols

from .....solvers.linear import *
from .....dynamics import *

months_list = ['January', 'February', 'March','April','May','June','July','August','September','October','November','December']

from  ...mechanics import display
import datetime

from dynpy.utilities.components.ode.en import ODEInitCodeComponent,ODEGeneralSolutionComponent,ODESteadySolutionComponent,ODESystemRepresentationComponent

miesiace_list = ['styczeń', 'luty', 'marzec','kwiecień','maj','czerwiec','lipiec','sierpień','wrzesień','październik','listopad','grudzień']

average_temp_list = [-1.9,-0.8,3.2,9.3,14.6,18,20.1,19.5,14.7,9.3,4.8,0.5]

Eg_daily_list_Wh_per_meter2 =[600,1000,3000,3800,4800,5400,5300,4900,3300,1700,700,500]

Eg_daily_list_kWh_per_meter2 = [0.6,1,3,3.8,4.8,5.3,4.9,3.3,1.7,0.7,0.5]

day_length_in_months_hours = [8.3,10.0,11.8,13.9,15.7,16.7,16.3,14.7,12.7,10.7,8.8,7.8]

data_atmospheric_conditions = {'Day length in a month [h]':day_length_in_months_hours,'Daily energy intensity [${kWh/m^2}$]':Eg_daily_list_Wh_per_meter2,'Average temperature [$^{\circ}$C]':average_temp_list}

df = pd.DataFrame(index = months_list,data = data_atmospheric_conditions)

class ReportComponent(BaseReportComponent):



    @property
    def reported_object(self):

        from .....solvers.linear import ODESystem
        
        if isinstance(self._reported_object, ODESystem):
            return self._reported_object

        else:
            return self._reported_object
            
    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj

    @property
    def _system(self):
        print('Kod do poprawienia #################### bo stosujesz starą zmienną self._system')
        print('zamień se self._system na self.reported_object')
        
        return self.reported_object



        
# #obsluga systemow dynamicznych


# imports_code_str = '''
# from sympy import *
# import numpy as np
# from pandas import *
# from sympy.physics.mechanics import init_vprinting, dynamicsymbols
# from pylatex import Document, Section, Subsection, Itemize, Package, HorizontalSpace, Description, Marker, Command 
# from pylatex.section import Paragraph, Chapter 
# from pylatex.utils import italic, NoEscape 
# from dynpy.utilities.adaptable import *
# from dynpy.utilities.templates.document import BeamerTemplate, MechanicalCase 
# from dynpy.utilities.templates.tikz import TikzCaSCStandalone
# from dynpy.utilities.report import (Markdown, Picture, SympyFormula, SymbolsDescription, DescriptionsRegistry, ObjectCode, CurrentContainer)
# from dynpy.solvers.linear import ODESystem
# init_vprinting()
# from dynpy.models.mechanics import SDOFWinchSystem

# '''
imports_code_str = '''
from sympy import *
import numpy as np

from dynpy.models.mechanics import SDOFWinchSystem

'''



class DynamicSystemCallComponent(ReportComponent):
    
    title="Introduction"


    def append_elements(self):

        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        display(ReportText('Required library/class imports to create the report:'))


        display(GuideCode(imports_code_str.replace('SDOFWinchSystem',system_name)))


        display(ReportText('A class that allows you to solve a specific problem (in this case, a problem related to the description of a system under consideration):'))

        display(GuideCode("""from dynpy.models.mechanics import SDOFWinchSystem""".replace('SDOFWinchSystem',system_name)      ))
        
        
        display(Markdown('Przypisujemy zaimportowany obiekt klasy do zmiennej o nazwie `dyn_sys` lub `system` w celu skrócenia jej zapisu'))
        
        display(GuideCode(f'system=dyn_sys={system_name}()'  ))

        display(ReportText('Path to the sought-after class on the CoCalc platform:'))
        #display(Picture('./dynpy/utilities/components/guides/images/sciezka_w.jpg'))

        display(ReportText((system.__class__.__module__)))
        
        display(ReportText('The way to call out the preview of the class - the so-called preview:'))

        display(GuideCode('''dyn_sys.as_picture()'''  ))
        display(system.as_picture())
        
        
class DynamicSystemMethodsUsageComponent(ReportComponent):

    title="Wywolywanie rownan ruchu oraz innych metod"



    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        #from dynpy.models.mechanics import ForcedSpringMassSystem as SDOFWinchSystem
        
        eoms=system._eoms[0]
        
        display(Markdown('''Definiujemy zmienną eoms w celu otrzymania równania w postaci wyrażenia sympy (Do raportu z obliczeniami),
            przypisujemy jej atrybut "_eoms[zmienna]" który wywołuje równanie ruchu
            w miejsce "zmienna" wpisujemy indeks równania ruchu, w przyadku równania o jednym stopniu swobody wpisujemy 0'
            '''))
        
        
        
        display(Markdown('Proces wywołowania równania ruchu za pomoca metody `_eoms`:'))
        display(GuideCode('''eoms_expr=dyn_sys._eoms[0]'''.replace('SDOFWinchSystem',system_name)))



        display(ReportText('Wynik jest następujący:'))
        display(SympyFormula(eoms))


        eoms_eq=Eq(eoms,0)
        eoms_eq
        
        display(Markdown('''Definiujemy zmienną odesys w celu przeprowadzenia obliczen
        obiekt odesys jest strukturą wyjaśnioną w guide basics of odesystems
        przypisujemy jej atrybut eoms który wywołuje równanie ruchu układu mechanicznego
        '''))

        display(GuideCode('''ode_sys=dyn_sys.eoms'''.replace('SDOFWinchSystem',system_name)))
        display(ReportText('Wynik jest następujący:'))
    
        display(SympyFormula(system.eoms))
        
        display(GuideCode('''ode_sys.as_eq_list()'''.replace('SDOFWinchSystem',system_name)))        
        
        display(ReportText('Proces tworzenia równania z wcześniej wywołanego równania ruchu (równanie początkowo jest wywoływane jako ciąg znaków reprezentujący opis danego problemu, jednak nie jest on wówczas równaniem, które możemy zastosować do obliczeń):'))
        display(GuideCode('eoms_eq=Eq(eoms,0)'.replace('SDOFWinchSystem',system_name)))
        display(ReportText('Wynik jest następujący:'))
        display(SympyFormula(eoms))

        #display(Picture('./Image/eomseq_w.jpg'))
        
        slownik=system.get_random_parameters()
        
        slownik_numerical=system.get_numerical_parameters()
        
        display(ReportText('Słownik z losowymi wartościami parametrów symbolicznych systemu wygląda następująco:'))
        
        display(SympyFormula(Dict(slownik)))
        
        display(ReportText('Słownik z losowymi wartościami liczbowymi do analizy numerycznej systemu, wygląda następująco:'))
        
        display(SympyFormula(Dict(slownik_numerical)))

        display(ReportText('Tworzenie słownika (w tym przypadku z losowymi wartościami) umożliwiający nadanie wartości zmiennym:'))
        
        

        display(GuideCode('slownik=SDOFWinchSystem().get_random_parameters()'.replace('SDOFWinchSystem',system_name)))
        display(GuideCode('slownik_numerical=SDOFWinchSystem().get_numerical_parameters()'.replace('SDOFWinchSystem',system_name)))
        #display(Picture('./Image/slownik_w.jpg'))

        display(ReportText('Rozwiązanie ogólne równania:'))
        #display(Picture('./Image/ogolne_w.jpg'))

#         solution=system._ode_system.steady_solution[0]
#         display(GuideCode('''solution=SDOFWinchSystem()._ode_system.solution[0]'''.replace('SDOFWinchSystem',system_name)))
#         display(GuideCode('solution_sym=SDOFWinchSystem()._ode_system.solution'.replace('SDOFWinchSystem',system_name)))
#         display(SympyFormula(solution))

#         display(ReportText('Rozwiązanie szczególne równania:'))



#         steady_solution=system._ode_system.steady_solution[0]
#         display(GuideCode('''steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]'''.replace('SDOFWinchSystem',system_name)))
#         display(SympyFormula(steady_solution))
    
steady_sol_str=(
'''

slownik_numerical=system.get_numerical_parameters()

steady_solution=SDOFWinchSystem()._ode_system.subs(slownik_numerical).steady_solution
steady_solution_subs=steady_solution

''')



solution_table_str=(
'''
coord = system.q[0]
table=table_eq.numerized().compute_solution(t_span)
table_new=table[[coord]]

''')

report_table_define_str=(
'''

tabela = Section('Table')
CurrentContainer(tabela)
coord = system.q[0]
solution_sym=ForcedSpringMassSystem()._ode_system.subs(slownik_numerical).steady_solution

general_sol_matrix=solution_sym
display(general_sol_matrix)
table_eq=general_sol_matrix.numerized(slownik_numerical).compute_solution(t_span)
table=table_eq

table_for_report = table.iloc[0:10].to_latex_dataframe()

table_for_report.reported(caption=('Table'))


''')


class SimulationsComponent(ReportComponent):
    
    title="Creating graphs"

    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__


        display(ReportText('Definition of time vector:'))
        #display(Picture('./Image/tspan_w.jpg'))

        display(GuideCode('''t_span = np.linspace(0,100,200)'''.replace('SDOFWinchSystem',system_name)))
        t_span = np.linspace(0,100,200)


        display(ReportText('The way to create charts manually:'))
        display(ReportText('Substitution of data from a previously created dictionary:'))
        #display(Picture('./Image/stedisub_w.jpg'))

        
        
        display(GuideCode(steady_sol_str.replace('SDOFWinchSystem',system_name)))
        #dict=system.get_random_parameters()
        dict_numerical=system.get_numerical_parameters()
        
        steady_solution=system._ode_system.steady_solution.rhs[0]
        steady_solution_subs=steady_solution.subs(dict_numerical)
        display(steady_solution_subs)
        display(ReportText('Using the lambdify method to convert functions to anonymous form'))
        #display(Picture('./Image/lambdify_w.jpg'))


#         eq_lambdify=lambdify(system.ivar, list(steady_solution_subs.rhs))
        eq_lambdify=lambdify(system.ivar, steady_solution_subs)
        display(GuideCode('''eq_lambdify=lambdify(system.ivar, list(steady_solution_subs.rhs))'''.replace('SDOFWinchSystem',system_name)))
        display(SympyFormula(eq_lambdify))


        display(ReportText('Creating a graph:'))
        #display(Picture('./Image/plot2_w.jpg'))
        
        #pd.DataFrame(data=eq_lambdify(t_span),index=t_span).plot()
        #plt.show
        df = pd.DataFrame(data=eq_lambdify(t_span))
        display(df.plot())
        display(GuideCode('''df = pd.DataFrame(data=eq_lambdify(t_span))
df.T.set_index(t_span).plot()'''.replace('SDOFWinchSystem',system_name)))


        display(ReportText('A way to create charts directly from a previously called class:'))
        display(ReportText('Dictionary substitution:'))
        #display(Picture('./Image/table_in.jpg'))

        
        #zmiana slownik na slownik_numerical
        table_eq=system._ode_system.steady_solution.subs(dict_numerical)


        display(GuideCode('''dict_numerical=system.get_numerical_parameters()
table_eq=system._ode_system.steady_solution.subs(dict_numerical)
table_eq'''.replace('SDOFWinchSystem',system_name)))
        display(SympyFormula(table_eq))

        #jak tabele zrobic + ew jej output
        display(ReportText('The way to create a table:'))
        #display(Picture('./Image/table_in.jpg'))
        coord = system.q[0]
        table=table_eq.numerized().compute_solution(t_span)
        table_new=table[coord]

        display(GuideCode(solution_table_str.replace('SDOFWinchSystem',system_name)))
        display(table)
        display(table_new)

        display(ReportText('The way to create a visualization of the defined table:'))
        #display(Picture('./Image/table_in.jpg'))

##### nazwy image tez przetlumaczone
        display(ReportText('How to create the table displayed in the finished report:'))
        #display(Picture('./Image/creating_table_in.png'))

        display(GuideCode(report_table_define_str.replace('SDOFWinchSystem',system_name)))

        display(ReportText('Visualization of the graph:'))
        #display(Picture('./Image/tikzplot_w.jpg'))

        display(GuideCode('''table_new.to_pylatex_tikz().in_figure()'''.replace('SDOFWinchSystem',system_name)))    

section_development_str=(
'''
from dynpy.utilities.report import ReportText
intro=Section('Introduction')
CurrentContainer(intro)

#scheme=Picture('picture_winch.png')
#display(scheme)

picture = Picture(system._as_picture().__dict__['image'], caption = f'System diagram')
display(picture)
''')

forming_equation_str=(
'''
eom=Section('Equations of motion')
CurrentContainer(eom)
display(ReportText('Equations of motion: '))
eoms=system._eoms[0]

slownik=system.get_random_parameters()
slownik_numerical=system.get_numerical_parameters()

eoms_eq=Eq(eoms,0)
solution_subs = system._ode_system.steady_solution.subs(slownik_numerical)
steady_solution=system._ode_system.steady_solution[0]
steady_solution_subs=steady_solution.subs(slownik_numerical)
#zmiana slownik na slownik numerical

steady_solution_subs
display(solution_subs)

ode_system = system._ode_system #ESystem.from_dynamic_system(system.linearized())
param_dict = system.get_numerical_parameters()
t_span = np.linspace(1,10,1001)
ic_list = [0.0,0.0] ### Dla układu o jednym stopniu swobody
ode_solution = ode_system.subs(param_dict).steady_solution

display(ode_solution)

display(SympyFormula(eoms_eq))
display(SympyFormula(solution_subs))
display(SympyFormula(steady_solution_subs))
''')

figure_eoms=(
'''
wykres=Section('Graph of dynamic system ')
CurrentContainer(wykres)
display(ReportText('The graph shows the solution of the equations of motion for the dynamic system under consideration. The graph shows the dependence of the displacement (or other relevant quantity, depending on the type of system) as a function of time, obtained from the solution of the differential equations.'))

ode_simulation = ode_solution.subs(param_dict).compute_solution(t_span, ic_list = ic_list)
picture=ode_simulation.to_pylatex_tikz(subplots=True).in_figure(caption='Graph of the solution of the equations of motion for the dynamic system under consideration.')

display(picture)

display('Equations of motion:')

display(eoms_eq)
display(solution_subs)
display(steady_solution_subs)
''')

report_generation_libraries=(
'''
doc_final = MechanicalCase('./output/Document',documentclass=NoEscape('article'),document_options=['a4paper','fleqn'],lmodern=False)

doc_final.append(intro)
doc_final.append(eom)
doc_final.append(wykres)

doc_final.generate_pdf()
''')




class DynSysCodeComponent(ReportComponent):

    title="Calling dynamic system code"



    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        display(ReportText('''The code of the analyzed class is as follows'''))
        display(ObjectCode(system.__class__))
        display(ReportText(f'''The presented code of the {system_name} class was retrieved with the help of the ObjectCode class.  '''))
        
        

        
class DynSysIntroComponent(ReportComponent):

    title="Introduction to using the DynSys module"
    
    def append_elements(self): 
        from dynpy.models.mechanics.tmac import SDOFWinchSystem
        
        display(ReportText('Required imports of libraries/individual classes to create the report:'))
        display(ObjectCode(
        '''
                from sympy import *
                import numpy as np
                from pandas import *
                from sympy.physics.mechanics import init_vprinting, dynamicsymbols
                from pylatex import Document, Section, Subsection, Itemize, Package, HorizontalSpace, Description, Marker, Command 
                from pylatex.section import Paragraph, Chapter 
                from pylatex.utils import italic, NoEscape 
                from dynpy.utilities.adaptable import *
                from dynpy.utilities.templates.document import BeamerTemplate, MechanicalCase 
                from dynpy.utilities.templates.tikz import TikzCaSCStandalone
                from dynpy.utilities.report import (SystemDynamicsAnalyzer, DataPlot, AccelerationComparison, FFTComparison, ReportEntry, SimulationalBlock,               ReportText, SimulationFFT, DataStorage, Markdown, SummaryTable, Picture, SympyFormula, SymbolsDescription, DescriptionsRegistry,
                ObjectCode, CurrentContainer)
                from dynpy.solvers.linear import ODESystem
                init_vprinting()
                from dynpy.models.mechanics.tmac import SDOFWinchSystem

        '''))
        display(ReportText('A class that allows you to solve a specific problem (in this case, a problem related to the description of a crane):'))
        display(Markdown('''
            from dynpy.models.mechanics.tmac import SDOFWinchSystem
            '''))

        display(ReportText('The way to call the class preview:'))

        display(ObjectCode('''
            SDOFWinchSystem().preview()
            '''))

  
        eoms=SDOFWinchSystem()._eoms[0]
        eoms

        display(ReportText('The process of calling the equation of motion using the eoms method:'))
        display(ObjectCode(
            '''
            eoms=SDOFWinchSystem()._eoms[0]
            eoms
            '''))

        display(ReportText('The result is:'))
        display(SympyFormula(eoms))


        eoms_eq=Eq(eoms,0)
        eoms_eq
        display(ReportText('The process of creating an equation from a previously called equation of motion (the equation is initially called as a string representing a description of the problem at hand, but it is not then an equation that we can use for calculations):'))
        display(ObjectCode('''
            eoms_eq=Eq(eoms,0)
            eoms_eq
            '''))
        display(ReportText('The result is:'))
        display(SympyFormula(eoms))

        slownik=SDOFWinchSystem().get_random_parameters()


        display(ReportText('Create a dictionary (in this case with random values) that allows you to assign values to variables:'))
        display(ObjectCode('''
            slownik=SDOFWinchSystem().get_random_parameters()
            slownik
            '''))
        display(slownik)

        display(ReportText('General solution of the equation:'))

        solution=SDOFWinchSystem()._ode_system.solution[0]
        display(ObjectCode(
            '''
                solution=SDOFWinchSystem()._ode_system.solution[0]
                solution

            '''))
        display(SympyFormula(solution))

        display(ReportText('Special solution of the equation:'))



        steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]
        display(ObjectCode(
            '''
                steady_solution=SDOFWinchSystem()._ode_system.steady_solution[0]
                steady_solution

            '''))
        display(SympyFormula(steady_solution))


class ODEReportComponent(ReportComponent):
    
    title="Stworzenie raportu"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('Aby utworzyć dokument, standardowo należy utworzyć sekcje lub rozdziały w następujący sposób:'))
#        display(Picture('./dynpy/utilities/components/guides/images/dok.jpg'))
        display(ObjectCode(ChapterContainer_str))
        display(ReportText('Jeżeli chcemy żeby nasze równania ładnie wyswietlały się w dokumencie nie możemy zapominac o użyciu SympyFormula:'))
        display(ObjectCode(Chapter_SympyFormula))
#        display(Picture('./dynpy/utilities/components/guides/images/sympaj.jpg'))
        display(ReportText('Co więcej, ODESystem posiada ogromną ilosć komponentów raportujących, które tworzą sekcje raportu za nas. Scieżka do ich odnalezienia jest następująca:'))
        display(ObjectCode(ode_components_path_str)) 
#        display(Picture('./dynpy/utilities/components/guides/images/sciezka_ode.jpg'))
        display(ReportText('Co więcej, istnieje też property .report, o którym informacja znajduje się rozdział niżej. Aby wygenerować plik pdf z użyciem wybranego szablonu należy zdefiniować dokuemnt jako wybrany template. Przypomnienie: Sprawdź, czy wszystkie katalogi w ścieżce poniżej istnieją. Jeśli nie, utwórz brakujące foldery przed próbą wygenerowania pliku PDF. Dobrym nawykiem jest stworzenie katalogu na pliki tekstowe wewnątrz swojego projektu. Następnie dodaj wszystkie sekcje używając metody append'))
        display(ObjectCode(property_report_formula)) 
#        display(Picture('./dynpy/utilities/components/guides/images/appeend.jpg'))

sym_num_system_code_str = ('''

m,F,c,k,x=symbols('m,F,c,k,x', positive=True)
t=Symbol('t')
x=Function(x)(t)

subs_dict={m:100 ,
          F:200 ,
          c:50 ,
          k:500,
          }

ics_list = [0.0,0.0]

''')



sym_harmonic_oscilator_eq = ('''

harmonic_oscilator_eq = Eq(m*x.diff(t,t) -F + c*x.diff(t) + k*x,0)
harmonic_oscilator_eq

''')

sym_harmonic_oscylator_ode = ('''

harmonic_oscylator_ode = ODESystem(odes= Matrix([(harmonic_oscilator_eq.lhs/m).expand()]),dvars = Matrix([x]),ode_order=2 )
harmonic_oscylator_ode

''')


sym_harmonic_oscylator_sol = ('''

harmonic_oscylator_sol = harmonic_oscylator_ode.numerized(subs_dict)
display(harmonic_oscylator_sol)
harmonic_oscylator_sym = harmonic_oscylator_sol.compute_solution(t_span=t_span, ic_list=ics_list)
display(harmonic_oscylator_sym)
harmonic_oscylator_sym.plot()

''')




sym_Example_PulledPendulum =(
'''

from dynpy.models.mechanics.pendulum import  PulledPendulum
t_span=np.linspace(0, 2, 1000)

dyn_sys_P = PulledPendulum()
display(dyn_sys_P._eoms)

subs_dict_P = dyn_sys_P.get_numerical_parameters()
display(subs_dict_P)
subs_dict_P = { **subs_dict_P, dyn_sys_P.F: 10, dyn_sys_P.Omega: 3.14/5}
for key, item in subs_dict_P.items():
    print(key)
    if key == dyn_sys_P.l**2*dyn_sys_P.m:
        key_to_pop  = key
subs_dict_P.pop(key_to_pop)
display(subs_dict_P)
ics_list_P = [0.0,0.0] # 1 stopien swobody

''')

sym_ode_sol_P_plot = ('''

ode_sol_P = dyn_sys_P._ode_system.numerized(subs_dict_P)

display(ode_sol_P)

sym_wyn_P = ode_sol_P.compute_solution(t_span=t_span, ic_list=ics_list_P)
sym_wyn_P.plot()

''')

sym_ode_sys_P =( '''

ode_sys_P = ODESystem.from_dynamic_system(dyn_sys_P.linearized())
ode_sys_P


''')

sym_ode_sol_P =( '''

ode_sol_P = ode_sys_P.solution.with_ics(ics_list)
ode_sol_P

''')

sym_num_sol_P=(
'''
num_sol_P = num_mod_P.compute_solution(t_span=t_span, ic_list=ics_list_P)

display(num_sol_P.plot())
''')

sym_coords=(
'''
display(harmonic_oscylator_ode)

display(harmonic_oscylator_sol)

coords= ode_sol_P.lhs
display(coords)
display(ode_sys_P)
display(subs_dict_P)
nadf = NumericalAnalysisDataFrame.from_model(ode_sys_P,F,[10, 20, 30], reference_data=subs_dict_P,coordinates=list(coords), index=pd.Index(t_span, name=t),ics=None)

nadf_sol = nadf.perform_simulations()

nadf_sol.columns = nadf_sol.columns.droplevel(0)
nadf_sol

''')

sym_wywolanie_helpa=(
'''
help(NumericalAnalysisDataFrame)
''')

sym_nadf=(
'''
nadf_sol.plot()
''')

sym_sms=(
"""
Omega = Symbol('Omega', positive=True)

subs_dict={m:10,
    F:100,
    c:0.01,
    k:500,
    }

t_span = np.linspace(1,2000,2001)

ode_sys_SMS = harmonic_oscylator_ode.subs({F: F*sin(Omega*t)}).subs({Omega: sqrt(k/m)+0.01})

ode_sys_SMS
""")

sym_num_wyn_SMSe=(
'''
num_wyn_SMS = ode_sys_SMS.subs(subs_dict).numerized().with_ics(ics_list).compute_solution(t_span)
''')

sym_ana_wyn_SMS=(
'''
ana_wyn_SMS = ode_sol_SMS.subs(subs_dict).with_ics(ics_list).compute_solution(t_span)
''')

sym_sms_plot=(
'''
display(num_wyn_SMS[x].plot())
''')

sym_comparison=(
"""
sym_comparison = pd.DataFrame(num_wyn_SMS[x])
sym_comparison[Symbol('x(t)_a')] = ana_wyn_SMS[x]

sym_comparison = TimeDataFrame(sym_comparison)
""")

sym_comparison_plot=(
'''
sym_comparison.to_frequency_domain().single_sided_rms().truncate(0,0.2).plot()
''')



sym_wyn_code=(
'''
sym_wyn_P = ode_sol_P.subs(subs_dict_P).compute_solution(t_span)
sym_wyn_P.plot()
''')



class ODENumericalSimulationsComponent(ReportComponent):
    
    title="Symulacje numeryczne- różne techniki symulacji"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('Przeprowadzenie symulacji przy pomocy ODE'))
        display(ReportText('W pierwszej kolejności zdefiniowano parametry potrzebne do przeprowadzenia do obliczeń'))

        display(ObjectCode(sym_num_system_code_str))

        display(ReportText('Wyprowadzenie wzoru na oscylator harmoniczny'))
        display(ObjectCode(sym_harmonic_oscilator_eq))
        #display(ObjectCode(harmonic_esc_str))

        display(ReportText('Przeprowadzenie symulacji w ODE'))


        display(ObjectCode(sym_harmonic_oscylator_ode))


        display(ObjectCode(sym_harmonic_oscylator_sol))

        #display(ObjectCode)

        #display(ObjectCode())

        display(ReportText('Wprowadzenie systemu dynamicznego, zdefiniowano również pojawiającą się Omegę '))
        display(ObjectCode(sym_Example_PulledPendulum))

        display(ObjectCode(sym_ode_sol_P_plot))
        display(ObjectCode(sym_ode_sys_P))
        display(ObjectCode(sym_ode_sol_P))

        display(ReportText('Symulacje używając systemu dynamicznego zdefiniowanego przy użyciu modułu dynpy'))

        display(ReportText('Sprawdzenie za pomocą helpa czym tak na prawdę jest NumericalAnalysisDataFrame'))
        display(ObjectCode(sym_wywolanie_helpa))

        display(ReportText('Wyświetlenie poszczególnych wykresów, wyników oraz tablicy'))
        display(ObjectCode(sym_coords))
        display(ObjectCode(sym_nadf))
        display(ReportText('Analiza porównawcza symulacji rozwiązania analitycznego i numerycznego'))
        display(ObjectCode(sym_sms))
        display(ObjectCode(sym_num_wyn_SMSe))
        display(ObjectCode(sym_ana_wyn_SMS))
        display(ObjectCode(sym_sms_plot))
        display(ObjectCode(sym_comparison))

        display(ObjectCode('''
        display(harmonic_oscylator_ode)

        display(harmonic_oscylator_sol)

        coords= harmonic_oscylator_sol.lhs

        nadf = NumericalAnalysisDataFrame.from_model(harmonic_oscylator_sol,F,[10, 20, 30], reference_data=subs_dict,coordinates=list(coords), index=pd.Index(t_span, name=t),ics=None)

        nadf_sol = nadf.perform_simulations()

        nadf_sol.columns = nadf_sol.columns.droplevel(0)

        nadf_sol
        '''))

        display(ObjectCode('''nadf_sol.plot()'''))




        display(ObjectCode('''
        Omega = Symbol('Omega', positive=True)

        subs_dict={m:10,
                   F:100,
                   c:0.01,
                   k:500,
                   }

        t_span = np.linspace(1,2000,2001)

        ode_sys_SMS = harmonic_oscylator_ode.subs({F: F*sin(Omega*t)}).subs({Omega: sqrt(k/m)+0.01})

        ode_sys_SMS
        '''))


        display(ObjectCode('''
        ode_sol_SMS = ode_sys_SMS.solution
        ode_sol_SMS.with_ics(ics_list)

        '''))

        display(ObjectCode('''num_wyn_SMS'''))


        display(ObjectCode('''ana_wyn_SMS'''))
        display(ObjectCode('''num_wyn_SMS_plot'''))
        display(ObjectCode('''ana_wyn_SMS_plot'''))

        display(ObjectCode('''
        sym_comparison = pd.DataFrame(num_wyn_SMS[x])
        sym_comparison[Symbol('x(t)_a')] = ana_wyn_SMS[x]

        sym_comparison = TimeDataFrame(sym_comparison)
        '''))

        display(ObjectCode('''
        sym_comparison.to_frequency_domain().single_sided_rms().truncate(0,0.2).plot()
        '''))

definicja_danych=(
'''
m=Symbol('m',positive=True)
g=Symbol('g')
alpha=Symbol('alpha')
v_0=Symbol('v_0')
t=Symbol("t")
x=Function("x")(t)
y=Function("y")(t)

dane = {m: 10, g: 9.81, alpha: pi/6, v_0:20}
''')

rownania_predkosci=(
'''
v_x=Eq(x.diff(t),v_0*cos(alpha))
v_y=Eq(y.diff(t),v_0*sin(alpha)-g*t)
display(v_x, v_y)
''')
definicja_ode=(
'''
ODE_x=ODESystem(odes=Matrix([v_x.lhs-v_x.rhs]),dvars=Matrix([x]),ode_order=1)
ODE_y=ODESystem(odes=Matrix([v_y.lhs-v_y.rhs]),dvars=Matrix([y]),ode_order=1)
display(ODE_x, ODE_y)
''')
ode_solution_code=(
'''
ODE_x_sol=ODE_x.subs(dane).solution.with_ics([20])
ODE_x_sol
''')
class ProjectileExampleComponent(ReportComponent):
    
    title="Przykład użycia ODESystem na podstawie rzutu ukośnego"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        from dynpy.solvers.linear import ODESystem
        m, g, v, alpha = symbols('m g v_0 alpha')

        t = Symbol('t')
        x = Function('x')(t)
        y = Function('y')(t)

        dane = {m: 10, g: 9.81, alpha: pi/6, v:20}
        predkosc_x=Eq(x.diff(t),v*cos(alpha))
        predkosc_y=Eq(y.diff(t),v*sin(alpha)-g*t)

        ode_x_x=ODESystem(odes=Matrix([predkosc_x.lhs-predkosc_x.rhs]),dvars=Matrix([x]),ode_order=1)
        ode_x_y=ODESystem(odes=Matrix([predkosc_y.lhs-predkosc_y.rhs]),dvars=Matrix([y]),ode_order=1)

        ODE_x_sol=ode_x_x.subs(dane).solution.with_ics([20])
        ODE_x_sol
        
        display(ReportText('Implementacja biblioteki ODESystem'))
        display(ObjectCode('''from sympy import *
from dynpy.solvers.linear import ODESystem
from sympy.physics.mechanics import dynamicsymbols'''))
  
        display(ReportText('Definiowanie zmiennych i utworzenie słownika'))
        display(ObjectCode(definicja_danych))
        
        display(ReportText('Równania ruchu dla osi x oraz y'))
        display(ObjectCode(rownania_predkosci))
        display(ReportText('Zapisanie równań za pomocą ODESystem'))
        display(ObjectCode(definicja_ode))

        display(ReportText('Wywołanie odpowiednio zmiennych i funkcji daje następujące rezultaty'))
        display(ObjectCode('''ODE_x'''))
        display(SympyFormula(ode_x_x))
        display(ObjectCode('''ODE_x.general_solution[0]'''))
        display(SympyFormula(ode_x_x.general_solution[0]))
        display(ObjectCode('''ODE_x.steady_solution[0]'''))
        display(SympyFormula(ode_x_x.steady_solution[0]))
        display(ObjectCode('''ODE_x.solution'''))
        display(SympyFormula(ode_x_x.solution[0]))
        display(ObjectCode('''ODE_x.solution.subs(dane)'''))

        display(SympyFormula(ode_x_x.solution.subs(dane)))#Fsimulations
        
        display(ObjectCode(ode_solution_code))

        display(SympyFormula(ODE_x_sol))
        display(ReportText('Ten sam wynik można osiągnąć stosując metoodę " dsolve ", jednak nie jest ona zalecana do równań różniczkowych'))
        display(ObjectCode('''dsolve(v_y.subs(dane))'''))
        display(SympyFormula(dsolve(predkosc_y.subs(dane))))

        
ode_import=(
'''
import numpy as np
import matplotlib.pylab as plt
import pandas as pd
from dynpy.solvers.linear import ODESystem
from sympy import Symbol, symbols, Function, Matrix
from dynpy.utilities.adaptable import *
''')

ode_variables=(
"""
t = Symbol('t')

U_z, R_w, L_w, E, U_Rw, U_Lw, k_e = symbols('U_z R_w L_w E U_Rw U_Lw k_e', positive=True)
M_s, B, J, M_obc, k_m = symbols('M_s B J M_obc k_m', positive=True)

i_w = Function('i_w')(t)
omega_s = Function('omega_s')(t)

dane = {
    U_z: 10,
    R_w: 2,
    L_w: 0.1,
    k_e: 0.1,
    k_m: 0.1,
    J: 0.1,
    B: 0.5,
    M_obc: 0
}

t_span = np.linspace(0, 1, 100)
""")

ode_objrctM=(
'''
ode = ODESystem(
    odes=Matrix([
        U_z - R_w * i_w - L_w * i_w.diff(t) - k_e * omega_s,
        k_m * i_w - J * omega_s.diff(t) - B * omega_s - M_obc
    ]),
    dvars=Matrix([i_w, omega_s]),
    ode_order=1
)
ode
''')

ode_met_1A=(
'''
ode.subs(dane).solution.with_ics([0,0])
''')

ode_tab_1A=(
'''
sym_an = ode.solution.with_ics([0,0]).subs(dane).numerized().compute_solution(t_span=t_span)
sym_an[[i_w, omega_s]]
''')

ode_met_1B=(
'''
sym_num = ode.subs(dane).numerized().compute_solution(t_span=t_span, ic_list=[0, 0])
sym_num[[i_w, omega_s]]
''')

ode_met_2=(
'''
X = [i_w, omega_s]

num_df = NumericalAnalysisDataFrame.from_model(
    ode.solution.with_ics([0, 0]),
    U_z,
    [10, 20, 50],
    reference_data=dane,
    coordinates=X,
    index=pd.Index(t_span, name=t)
)
num_df
''')

ode_met_PS=(
'''
sym_wyn = num_df.perform_simulations().droplevel(0,axis=1)

display(sym_wyn)
''')

ode_import_example=(
'''
from dynpy.models.mechanics.trolley import SpringDamperMassSystem

dyn_sys = SpringDamperMassSystem()

dyn_sys._ode_system
''')

ode_example2=(
'''
dane = dyn_sys.get_numerical_parameters()

data_dict = {**dane, dyn_sys.F: 1000}

data_dict
''')

ode_simulation=(
'''
simulation = dyn_sys._ode_system.subs(data_dict).numerized().compute_solution(
    np.linspace(0, 2, 1000), ic_list=[10, -5]
)

simulation
''')

ode_simulation_plot=(
'''
simulation.plot()
plt.grid()
plt.show()
''')




        
class ODESimulationComponent(ReportComponent):
    
    title="Przeprowadzanie symulacji"
    


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('To od czego standardowo powinno się zacząć to importy oraz definicja zmiennych. Potrzebne importy:'))
        display(ObjectCode(ode_import))
        display(ReportText('Definiowanie zmiennych:'))
        display(ObjectCode(ode_variables))
        display(ReportText('Kolejny niezbędny krok to tworzenie obiektu ODE. Należy go wykonać w następujący sposób:'))
        display(ObjectCode(ode_objrctM))
        display(ReportText('METODA 1A: Przeprowadzenie symulacji na gotowm rozwiazaniu analitycznym poprzez użycie property solution. Samo wywołanie                                 odpowiedniego rozwiązania analitycznego pokazuje nam poniższy kod:'))
        display(ObjectCode(ode_met_1A))
        display(ReportText('Następnie generujemy tabele niezbędną do symulacji w taki sposób:'))
        display(ObjectCode(ode_tab_1A))
        display(ReportText('METODA 1B: Innym sposobem jest przeprowadzenie symulacji numerycznych bez obliczeń analitycznych. Potrzebny jest do tego taki kod: '))
        display(ObjectCode(ode_met_1B))
        display(ReportText('METODA 2: Trzecim sposobem generowania symulacji jest użycie NumericalAnalysisDataFrame, kod wywołania prezentuje się następująco:'))
        display(ObjectCode(ode_met_2))
        display(ReportText('Aby przeprowadzić symulacje konieczne jest zastosowanie metody perform simulations. Bez zastosowania tej metody dostaniemy następujący wynik:'))
        display(ReportText('A z jej użyciem, wszystko powinno zadziałać i dostaniemy następujący output:'))
        display(ObjectCode(ode_met_PS))
        display(ReportText('Dla lepszego zrozumienia zagdnień poniżej przedstawione zostało zastosowanie symulacji na konkretnym przykładzie. Skorzystamy z                           prostej klasy SpringDamperMassSystem. Na dobry początek należy zaimportować tę klasę oraz przypisać ją do zmiennej. W przeprowadzeniu                                     symulacji pomoże nam wykorzystanie metody $ode system$ w celu pozyskania równania ruchu naszego systemu dynamicznego. Aby pozyskać dane w                                 prosty sposób korzystamy z metody $get numerical parameters$ oraz wrzucamy te dane w słownik. Kod do wykonania tych czynnosci prezentuje się                             następująco: '))
        display(ObjectCode(ode_import_example))
        display(ReportText('Następnie stosując jedną z wyżej pokazanych metod, możemy stworzyć tabelę:'))
        display(ObjectCode(ode_example2))
        display(ReportText('Ostatnim krokiem jest wizualizacja danych, czyli stworzenie wykresu. Aby taki wykres uzyskać wystarczy wpisać kod widoczny na                             obrazku:'))
        display(ObjectCode(ode_simulation))
        display(ObjectCode(ode_simulation_plot))


class_call_str=(
'''
from dynpy.models.mechanics.principles import ComposedSystem
from sympy import *

class MyMaterialPointMovement(ComposedSystem):
    pass
''')

atributes_str=(
'''
m = Symbol('m', positive=True)
g = Symbol('g', positive=True)
c = Symbol('c', positive=True)
r = Symbol('r', positive=True)
phi = dynamicsymbols('phi')

c0 = Symbol('c0', positive=True)
r0 = Symbol('r0', positive=True)
phi0 = dynamicsymbols('phi0')
''')

innit_str=(
'''
def __init__(self,
             m=None,
             g=None,
cLatexDataFrame.set_default_units(unit_dict)

tabelka=LatexDataFrame.formatted(
    data=dane,
    index=['Value']).applymap(lambda x: f'${latex(x)}$').rename_axis(
        'Symbol',
        axis=1)

display(tabelka.reported(caption='Data given in the exercise'))

''')

component_str=(
'''
@property
def components(self):

    components = {}


    self._mass_x = MaterialPoint(self.m,
                                 pos1=self.r * sin(self.phi),
                                 qs=self.qs)
    self._mass_y = MaterialPoint(self.m,
                                 pos1=self.r * cos(self.phi),
                                 qs=self.qs)

    self._gravity_ = GravitationalForce(self.m,
                                        self.g,
                                        pos1=self.r * cos(self.phi),
                                        qs=self.qs)



    components['_mass_x']=self._mass_x
    components['_mass_y']=self._mass_y
    components['_gravity_']=self._gravity_


    return components
''')


symb_desc_str=(
'''
def symbols_description(self):
    self.sym_desc_dict = {
        self.m: r'Mass',
        self.g: r'Gravity constant',
        self.c: r'',
    }

    return self.sym_desc_dict
''')

def_data_str=(
'''
def get_default_data(self):

    m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

    default_data_dict = {
        self.m: [m0 * no for no in range(1, 8)],
        self.c: [c0 * no for no in range(1, 8)],
        self.r: [r0 * no for no in range(1, 8)],
        self.phi: [phi0 * no for no in range(1, 8)],
    }

    return default_data_dict
''')

num_data_str=(
'''
def get_numerical_data(self):

    m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

    default_data_dict = {
        self.m: [m0 * no for no in range(1, 8)],
        self.c: [c0 * no for no in range(1, 8)],
        self.r: [r0 * no for no in range(1, 8)],
        self.phi: [phi0 * no for no in range(1, 8)],
    }

    return default_data_dict
''')

dict_str=(
'''
def unit_dict(self):

    from pint import UnitRegistry
    ureg=UnitRegistry()

    unit_dict = {
        self.m: ureg.kilogram,
        self.g: ureg.meter/ureg.second/ureg.second,
        self.c: ureg.kilogram/ureg.second,
        self.r: ureg.meter,
        self.phi: ureg.radian,
        self.c0: ureg.kilogram/ureg.second,
        self.r0: ureg.meter,
        self.phi0:  ureg.radian
    }
    return unit_dict
''')

full_class_str=(
'''
from dynpy.models.mechanics.principles import ComposedSystem
from sympy import *

class MyMaterialPointMovement(ComposedSystem):

    m = Symbol('m', positive=True)
    g = Symbol('g', positive=True)
    c = Symbol('c', positive=True)
    r = Symbol('r', positive=True)
    phi = dynamicsymbols('phi')

    c0 = Symbol('c0', positive=True)
    r0 = Symbol('r0', positive=True)
    phi0 = dynamicsymbols('phi0')

    def __init__(self,
                 m=None,
                 g=None,
                 c=None,
                 r=None,
                 phi=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if g is not None: self.g = g
        if c is not None: self.c = c
        if r is not None: self.r = r
        if phi is not None: self.phi = phi
        self.ivar = ivar

        self.qs = [self.phi]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}


        self._mass_x = MaterialPoint(self.m,
                                     pos1=self.r * sin(self.phi),
                                     qs=self.qs)
        self._mass_y = MaterialPoint(self.m,
                                     pos1=self.r * cos(self.phi),
                                     qs=self.qs)

        self._gravity_ = GravitationalForce(self.m,
                                            self.g,
                                            pos1=self.r * cos(self.phi),
                                            qs=self.qs)


      
        components['_mass_x']=self._mass_x
        components['_mass_y']=self._mass_y
        components['_gravity_']=self._gravity_
     

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass',
            self.g: r'Gravity constant',
            self.c: r'',
        }

        return self.sym_desc_dict

    def get_default_data(self):

        m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

        default_data_dict = {
            self.m: [m0 * no for no in range(1, 8)],
            self.c: [c0 * no for no in range(1, 8)],
            self.r: [r0 * no for no in range(1, 8)],
            self.phi: [phi0 * no for no in range(1, 8)],
        }

        return default_data_dict

    def get_numerical_data(self):

        m0, c0, r0, phi0 = self.m0, self.c0, self.r0, self.phi0

        default_data_dict = {
            self.m: [m0 * no for no in range(1, 8)],
            self.c: [c0 * no for no in range(1, 8)],
            self.r: [r0 * no for no in range(1, 8)],
            self.phi: [phi0 * no for no in range(1, 8)],
        }

        return default_data_dict
        
    def unit_dict(self):

        from pint import UnitRegistry
        ureg=UnitRegistry()

        unit_dict = {
            self.m: ureg.kilogram,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.c: ureg.kilogram/ureg.second,
            self.r: ureg.meter,
            self.phi: ureg.radian,
            self.c0: ureg.kilogram/ureg.second,
            self.r0: ureg.meter,
            self.phi0:  ureg.radian
        }
        return unit_dict        
''')

class DynSysImplementationComponent(ReportComponent):
    
    title="Implementacja systemow dynamicznych"


    def append_elements(self):
        
        #system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        #przekopiowac tu fragment kodu z guide pamietajac o indentach
        display(ReportText("Tworzenie klasy należy zacząć od wybrania nazwy opisującej nasz system oraz zadeklarowania jej w odpowiedni sposób, który został przedstawiony poniżej:")) #itp
        display(GuideCode(class_call_str))
        display(ReportText('Następnie deklarujemy atrybuty/symbole, które są niezbędne do opisania danego obiektu'))
        display(GuideCode(atributes_str))
        display(ReportText('Do zdefiniowania inicjalizacji obiektu klasy z określonymi atrybutami służy poniższa struktura.'))
        display(GuideCode(innit_str))
        display(ReportText('W tym kroku korzystamy z predefiniowanych elementów takich jak punkty materialne, dyski, itp.'))
        display(GuideCode(component_str))
        display(ReportText('W tej części opisujemy parametry klasy, np. masę'))
        display(GuideCode(symb_desc_str))
        display(ReportText('Sekcja ta przedstawia inicjalizację słownika zawierającego dane symboliczne do równań analitycznych systemu:'))
        display(GuideCode(def_data_str))
        display(ReportText('Kolejnym krokiem jest przypisanie słowników zawierających dane do symulacji numerycznych:'))
        display(GuideCode(num_data_str))
        display(ReportText('Ta sekcja opisuje definiowanie jednostek używanych przez system dynamiczny:'))
        display(GuideCode(dict_str))
        
        display(ReportText('Ostatecznie kod implementacji całego systemu ma następującą postać:'))
        display(GuideCode(full_class_str))


doc_gen_str =(
'''

doc_final = MechanicalCase('./output/nazwa_dokumentu',documentclass=NoEscape('article'),document_options=['a4paper','fleqn'],lmodern=False)
doc_final.packages.append(Package('natbib', options=['numbers']))
doc_final.packages.append(Package('booktabs'))
doc_final.packages.append(Package('float'))
doc_final.packages.append(Package('siunitx'))


doc_final.append(sekcja)
doc_final.append(sekcja1)
doc_final.append(sekcja2)
doc_final.append(sekcja3)
doc_final.append(sekcja4)
doc_final.append(sekcja5)

doc_final.generate_pdf()

''')

class DocumentGenerationComponent(ReportComponent):

    #title="Generowanie dokumentu"
    title="Document generation"


    @property
    def reported_object(self):


        default_data = {'classname':'ReportingModuleIntroComponent',
                       'module':'guide.en.py',
                       'field':'guide or report',
                       'target':'`ODESystem` class',
                       'issue_no':359,
                       }


        if isinstance(self._reported_object, dict):
            return {**default_data,**self._reported_object}

        elif isinstance(self._reported_object, str):
            return {**default_data,'classname':self._reported_object}

        elif self._reported_object is None:
            return default_data

        elif self._reported_object is None or not isinstance(self._reported_object, dict):
            return default_data
        
        else:
            return self._reported_object


    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj


    def append_elements(self):
        #variables provided by `reported_object` arg
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']
        
       
        #implement reporting activieties here
        
        #display(ReportText('Ostatni krok to zaapendowanie sekcji i utworzenie dokumentu pdf :'))
        display(ReportText('The last step is to append a section and create a pdf document :'))
        display(GuideCode(f'{doc_gen_str}'))




SympyFormulaComponent_str=(
'''
from sympy import *

M = Symbol('M')

t = Symbol('t')

m_e = Symbol('m_e')

e = Symbol('e')

z = Function('z')

varphi = Function('varphi')

Ek = M*Derivative(z(t), t)**2/2 + m_e*(e*sin(varphi(t))*Derivative(varphi(t), t) - Derivative(z(t), t))**2/2

display(Eq(Symbol('T'),Ek))
''')


class SympyFormulaComponent(DocumentGenerationComponent):

    title = "Klasa SympyFormula i jej zastosowanie"



    def append_elements(self):
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']

        #implement reporting activieties here
        display(ReportText('Przykładowe zastosowanie komponentu do obliczeń symbolicznych, na przykładzie energii kinetycznej:'))
        display(GuideCode(f'{SympyFormulaComponent_str}'))


ODEIcsComponent_dec_str = ('''
m, c, k, F, mu = symbols('m c k F mu', positive = True)
t = Symbol('t')
x = Function('x')(t)

subs_data = {m: 10, c: 10, k: 10, F: 5, mu: 2}

ode_1 = ODESystem(odes = Matrix([m*x.diff(t,t) + c *x.diff(t)  + k * x]), odes_rhs = Matrix([F*sin(mu * t)]), dvars = Matrix([x]))
display(ode_1)
''')
        
class ODEIcsComponent(ReportComponent):

    title = "ODESystem solution with set of initial conditions (ICS) component presentation"

    def append_elements(self):

        display(ReportText('First declare ODESystem component:'))
        
        display(ObjectCode(ODEIcsComponent_dec_str))
        
        from dynpy.solvers.linear import AnalyticalSolution, ODESolution, ODESystem
        import sympy
        
        m, c, k, F, mu = symbols('m c k F mu', positive = True)
        t = Symbol('t')
        x = Function('x')(t)

        subs_data = {m: 10, c: 10, k: 10, F: 5, mu: 2}

        ode_1 = ODESystem(odes = Matrix([m*x.diff(t,t) + c *x.diff(t)  + k * x]), odes_rhs = Matrix([F*sin(mu * t)]), dvars = Matrix([x]))
        
        display(ReportText('After executing above code bellow equation will appear'))
        
        display(ode_1)

        display(ReportText('To present solution with initial set of conditions, first you need to calculate solution:'))
        
        display(ObjectCode('sol_sym_1_pre_ics = ode_1.subs(subs_data).solution'))
        
        sol_sym_1_pre_ics = ode_1.subs(subs_data).solution
        
        display(sol_sym_1_pre_ics)
        
        display(ReportText('To calculate solution with ICS, declare ics vector, and run with_ics method as shown bellow:'))
        
        display(ObjectCode('''
ics = [1, 2]
sol_sym_1 = sol_sym_1_pre_ics.with_ics(ics)
display(sol_sym_1)'''))
        
        ics = [1, 2]
        sol_sym_1 = sol_sym_1_pre_ics.with_ics(ics)
        display(sol_sym_1)
        
ODEIcsCodeComponent_str =('''
m, c, k, F, mu = symbols('m c k F mu', positive = True)
t = Symbol('t')
x = Function('x')(t)

subs_data = {m: 10, c: 10, k: 10, F: 5, mu: 2}

ode_1 = ODESystem(odes = Matrix([m*x.diff(t,t) + c *x.diff(t)  + k * x]), odes_rhs = Matrix([F*sin(mu * t)]), dvars = Matrix([x]))

sol_sym_1_pre_ics = ode_1.subs(subs_data).solution

ics = [1, 2]

sol_sym_1 = sol_sym_1_pre_ics.with_ics(ics)
''')

class ODEIcsCodeComponent(ReportComponent):

    title = "ODESystem solution with set of initial conditions (ICS) component presentation"

    def append_elements(self):

        display(ReportText('For bellow ODESystem component example:'))
        
        display(ObjectCode(ODEIcsCodeComponent_str))
        
        from dynpy.solvers.linear import AnalyticalSolution, ODESolution, ODESystem
        import sympy
        
        m, c, k, F, mu = symbols('m c k F mu', positive = True)
        t = Symbol('t')
        x = Function('x')(t)

        subs_data = {m: 10, c: 10, k: 10, F: 5, mu: 2}

        ode_1 = ODESystem(odes = Matrix([m*x.diff(t,t) + c *x.diff(t)  + k * x]), odes_rhs = Matrix([F*sin(mu * t)]), dvars = Matrix([x]))

        sol_sym_1_pre_ics = ode_1.subs(subs_data).solution

        ics = [1, 2]

        sol_sym_1 = sol_sym_1_pre_ics.with_ics(ics)
        
        display(ReportText('To get call code for ICS, run bellow method:'))
        
        display(ObjectCode("sol_sym_1.get_ics()"))
        
        display(sol_sym_1.get_ics())
        


ds_overview_str=("""
from dynpy.utilities.documents.document import DynSysOverviewReport
from dynpy.utilities.components.guides.en import *
from dynpy.models.mechanics.pendulum import Pendulum

DynSysOverviewReport(reported_object=Pendulum());
""")

dynsys_comp_check_str = '''from dynpy.models.mechanics import ForcedSpringMassSystem

comp = ForcedSpringMassSystem()

DynamicSystemCompletenessCheckComponent(comp)'''

basicsymcomp_str = '''from dynpy.models.mechanics.tmac import SDOFWinchSystem

comp = SDOFWinchSystem()

BasicSymComponent(comp)'''

coderefactorcomp_str = '''from dynpy.utilities.components.guides.en import CodeRefactorComponent

CodeRefactorComponent(None)'''

dynsysimpcomp_str = '''from dynpy.utilities.components.guides.pl import DynSysImplementationComponent

DynSysImplementationComponent(None)'''

diffsimcomp_str = '''from dynpy.utilities.components.guides.en import DifferentSimulationsComponent

comp = SDOFWinchSystem()

DifferentSimulationsComponent(comp)'''

dynintrocomp_str = '''from dynpy.utilities.components.guides.en import *
DynSysIntroComponent(None)'''

dynsumcomp_str = '''from dynpy.utilities.components.guides.en import *

DynSysSummaryComponent(None)'''

dynmetuscomp_str = '''from dynpy.utilities.components.guides.en import DynamicSystemMethodsUsageComponent

comp = SDOFWinchSystem()

DynamicSystemMethodsUsageComponent(comp)'''
dyncompov_str = '''Guide consist of bellow components which can be called separatelly with most of avaible dynsys models:
*DynamicSystemCallComponent
*NumericalAnalysisSimulationComponent
*AnalyticalSimulationComponent
*DynSysCodeComponent
*IssuePreparationComponent'''

class DynSysOverviewUsageComponent(ReportComponent):

    title = "Introduction to usage of DynSysOverviewReport"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers in-deepth analysis of exsisting dynamic system that are basic elements of `DynPy` library.'))
        display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(ds_overview_str))
        
        display(ReportText(dyncompov_str))
        
        #display(ReportText('This guide concers in-deepth analysis of exsisting dynamic system that are basic elements of `DynPy` library.'))
        #display(ReportText('It can be run with any available dynamic system.'))
        '''
        #DynamicSystemCompletenessCheckComponent
        display(ReportText('This component calls and checks all key elements of the dynamic system'))
        display(ReportText('It can be run with any available dynamic systems.'))
        display(ObjectCode(dynsys_comp_check_str))
        
        #BasicSymComponent
        display(ReportText('This component shows how to do basic simulations for dynamic system.'))
        display(ReportText('It can be run for SDOFWinchSystem, as shown on below example call.'))
        display(ObjectCode(basicsymcomp_str))
        
        #CodeRefactorComponent
        display(ReportText('This component shows how to refactor DynSys component according to latest standard.'))
        display(ReportText('It is called without any argument.'))
        display(ObjectCode(coderefactorcomp_str))
        
        #DifferentSimulationsComponent
        display(ReportText('This component explains how to implement dynamic system with ComposedSystem class'))
        display(ReportText('It is called without any argument.'))
        display(ObjectCode(diffsimcomp_str))
        
        #DynSysImplementationComponent
        display(ReportText('This component explains how to implement dynamic system with ComposedSystem class'))
        display(ReportText('It can be run for SDOFWinchSystem, as shown on below example call.'))
        display(ObjectCode(dynsysimpcomp_str))
        
        #DynSysIntroComponent
        display(ReportText('This component is simple introduction to using the DynSys module'))
        display(ReportText('It can be run for SDOFWinchSystem, as shown on baeow example call.'))
        display(ObjectCode(dynintrocomp_str))
            
        #DynSysSummaryComponent
        display(ReportText('This component shows idea of code refactoring and additionally displays list of available dynamical systems'))
        display(ReportText('It is called without any argument.'))
        display(ObjectCode(dynsumcomp_str))
        
        #DynamicSystemMethodsUsageComponent
        display(ReportText('This component shows equations of motion and other methods avaible on DynSys model'))
        display(ReportText('It can be run with any available dynamic systems.'))
        display(ObjectCode(dynmetuscomp_str))
        '''


class DynamicSystemCompletenessCheckComponent(ReportComponent):

    title="Wywoływanie i sprawdzanie wszystkich kluczowych elementów systemu dynamicznego"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        eoms=system._eoms[0]
        eoms

        display(ReportText('Wywołanie systemu w pierwszej kolejności można sprawdzić przez sprawdzenie równania ruchu metodą _eoms.'))

        display(ReportText('Wynik jest następujący:'))
        
        display(Markdown(
            '''
        eoms_eq=Eq(eoms,0)
        eoms_eq
            '''))
        
        eoms_eq=Eq(eoms,0)
        eoms_eq
        
        display(SympyFormula(eoms_eq))
        
        display(ReportText('Następnie wykonuje się analizę metody __init__ klasy:'))
        
        display(ObjectCode(system.__init__))
        
        display(ReportText('Kolejnym etapem jest sprawdzenie, czy schemat oraz zdjęcie reczywiste systemu są zdefiniowane:'))
        
        display(Markdown(
            '''
        system._as_picture()
        system.preview(example = 'detail_real_name')
            '''))
        
        system._as_picture()
        system.preview(example = 'detail_real_name')
        
        display(ReportText('Kolejnym etapem jest sprawdzenie, czy zdefiniowane są słowniki z parametrami:'))
        
        display(Markdown(
            '''
        system.get_random_parameters()
        system.get_numerical_parameters()
            '''))
        
        display(ReportText('Jeśli podczas wywołania pojawia się błąd o braku tej metody lub parametry są inne niż w metodzie __init__ należy zmodyfikować klasę analogicznie do pokazanego rozwiązania:'))
        
        display(Markdown(
            '''
        
        def get_default_data(self):

            m0, k0 = self.m0, self.k0

            default_data_dict = {

                self.m: [S.One * no * m0 /100 for no in range(80, 120)], # percentage
                self.k_r: [S.One * no * k0/100 for no in range(80, 120)], # percentage
                self.k_l: [S.One * no * k0/100 for no in range(70, 110)], # percentage
            }
            return default_data_dict

        def get_numerical_data(self):


            m0, k0 = 100, 1000


            default_data_dict = {
                self.m: [S.One * no * m0 /100 for no in range(80, 120)], # percentage
                self.k_r: [S.One * no * k0/100 for no in range(80, 120)], # percentage
                self.k_l: [S.One * no * k0/100 for no in range(70, 110)], # percentage

            }
            return default_data_dict
            '''))
        
        system.get_random_parameters()
        system.get_numerical_parameters()
        
        display(ReportText('Następnie konieczne jest sprawdzenie opisów symboli oraz jednostek:'))
        
        display(Markdown(
            '''
        system.symbols_description()
        system.unit_dict()
            '''))
        
        display(ReportText('Jeśli podczas wywołania tej metody pojawia się błąd, należy wykonać modyfikację w analogizny sposób do:'))
        
                
        display(Markdown(
            '''
        def symbols_description(self):
            self.sym_desc_dict = {
                self.m: r'system's mass',
                self.k_l: r'left spring stiffness',
                self.k_r: r'left spring stiffness',
            }
            return self.sym_desc_dict

        def unit_dict(self):
        
            from pint import UnitRegistry
            ureg=UnitRegistry()

            unit_dict = {
                self.m: ureg.kilogram,
                self.k_l: ureg.Newton/ureg.meter,
                self.k_r: ureg.Newton/ureg.meter,
            }
            return unit_dict

            '''))
        
        from pint import UnitRegistry
        ureg=UnitRegistry()
        
        system.symbols_description()
        system.unit_dict()


class DynSysSummaryComponent(ReportComponent):

    title = "DynSys system code refactor component"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('DynSys system consists of the following models:'))
        
        mods=inspect.getmembers(mechanics, inspect.ismodule)

        for nam,mod in mods:

            mods_tmp = [name for name,cls in inspect.getmembers(mod, inspect.isclass) if issubclass(cls,mechanics.principles.ComposedSystem)]
            classes_str=',\n - '.join(mods_tmp)
            if mods_tmp != []:
                display(ReportText(f' {nam}'))
                display(ReportText(f'- {classes_str}'))
                
        display(ReportText('All of this models should be refactored according to bellow example:'))
        CodeRefactorComponent(None)

class DynSysCompletenessCheckerVerbosedComponent(ReportComponent):

    title="Checking completeness of dynamic systems"



    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        display(ReportText('''Bellow table present result of completeness check of givem dynamic systems'''))
        
        from dynpy.utilities.tools import DynsysCheckerTable
        
        display(DynsysCheckerTable(system, verbosity= True).get_table())

class DynSysCompletenessCheckerComponent(ReportComponent):

    title="Checking completeness of dynamic systems"

    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        system_name = system.__class__.__name__
        
        display(ReportText('''Bellow table present result of completeness check of givem dynamic systems'''))
        
        from dynpy.utilities.tools import DynsysCheckerTable
        
        display(DynsysCheckerTable(system).get_table())
        
ode_overview_str=("""
from dynpy.utilities.documents.document import ODESystemOverviewReport
from dynpy.utilities.components.guides.en import *
from dynpy.models.mechanics.pendulum import Pendulum

ODESystemOverviewReport(reported_object=Pendulum());

""")

odecompsov_str =('''Guide consist of bellow components which can be called separatelly with most of avaible ODESystem models:
*ODEInitCodeComponent
*ODEGeneralSolutionComponent
*ODESteadySolutionComponent
*ODESystemRepresentationComponent
*ODESystemExponentiationComponent
*PredictionOfSteadySolutionComponent
*HomoPredictionIntroComponent
*MainPredictionComponent
*RootsAnalysisComponent
''')

class ODESystemOverviewUsageComponent(ReportComponent):

    title = "Introduction to usage of ODESystemOverviewReport"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers in-deepth analysis of ODESystem solver capabilities that are crucial for simulating system from `DynPy` library.'))
#         display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(ds_overview_str))
        
        display(ReportText(odecompsov_str))

        
basic_dyn_sys_impl_intro_call_str=("""
from dynpy.utilities.documents.guides import BasicsOfDynSysImplementationGuide
from dynpy.utilities.components.guides.en import *
BasicsOfDynSysImplementationGuide();
""")


basic_dsys_impl_intro_str =('''Guide consist of bellow components which can be called separatelly:
*BasicsOfDynSysImplementationIntroComponent
*DynSysImplementationComponent
*DynamicSystemCallComponent
*DynamicSystemMethodsUsageComponent
*SimulationsComponent
*DynSysCodeComponent
''')

class BasicsOfDynSysImplementationIntroComponent(ReportComponent):

    title = "Introduction to basics of dynamic systems implementation"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers basics of structure and implementation of dynamic systems in `DynPy` library.'))
        display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(basic_dyn_sys_impl_intro_call_str))
        
        display(ReportText(basic_dsys_impl_intro_str))
        
        
ODE_sys_intro_call_str=("""
from dynpy.utilities.documents.guides import BasicsOfODESystemGuide
from dynpy.utilities.components.guides.en import *
from dynpy.models.mechanics.pendulum import ForcedSpringMassSystem

BasicsOfODESystemGuide(reported_object=ForcedSpringMassSystem());

""")


ODE_sys_intro_str =('''Guide consist of bellow components which can be called separatelly:
*ODESystemsUsageIntroComponent
*BasicUsageOfODESystemComponent
*ReportCompUseComponent
*ProjectileExampleComponent
*ODESimulationComponent
*ODENumericalSimulationsComponent
''')

class ODESystemsUsageIntroComponent(ReportComponent):

    title = "Introduction to basics of ODE systems"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers basics of structure and usage of ODE systems in  `DynPy` library.'))
        display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(ODE_sys_intro_call_str))
        
        display(ReportText(ODE_sys_intro_str))
        
        
dyn_sys_intro_call_str=("""
from dynpy.utilities.documents.guides import UsageOfDynamicSystemsGuide
from dynpy.utilities.components.guides.en import *
from dynpy.models.mechanics.pendulum import ForcedSpringMassSystem

UsageOfDynamicSystemsGuide(reported_object=ForcedSpringMassSystem());

""")


dsys_intro_str =('''Guide consist of bellow components which can be called separatelly:
*DynamicSystemsUsageIntroComponent
*DynamicSystemCallComponent
*DynamicSystemMethodsUsageComponent
*NumericalAnalysisSimulationComponent
*SimulationsComponent
*SimulationReportComponent
''')

class DynamicSystemsUsageIntroComponent(ReportComponent):

    title = "Introduction to basics of dynamic systems"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers basics of structure and usage of dynamic systems in  `DynPy` library.'))
        display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(dyn_sys_intro_call_str))
        
        display(ReportText(dsys_intro_str))


from  ...mechanics import *
from  ...mechanics import ReportComponent as BaseReportComponent


from . import pl

import pandas as pd
import numpy as np
from sympy import lambdify

from sympy import *
from pandas import *
from sympy.physics.mechanics import dynamicsymbols

from .....solvers.linear import *
from .....dynamics import *

months_list = ['January', 'February', 'March','April','May','June','July','August','September','October','November','December']

from  ...mechanics import display
import datetime


miesiace_list = ['styczeń', 'luty', 'marzec','kwiecień','maj','czerwiec','lipiec','sierpień','wrzesień','październik','listopad','grudzień']

average_temp_list = [-1.9,-0.8,3.2,9.3,14.6,18,20.1,19.5,14.7,9.3,4.8,0.5]

Eg_daily_list_Wh_per_meter2 =[600,1000,3000,3800,4800,5400,5300,4900,3300,1700,700,500]

Eg_daily_list_kWh_per_meter2 = [0.6,1,3,3.8,4.8,5.3,4.9,3.3,1.7,0.7,0.5]

day_length_in_months_hours = [8.3,10.0,11.8,13.9,15.7,16.7,16.3,14.7,12.7,10.7,8.8,7.8]

data_atmospheric_conditions = {'Day length in a month [h]':day_length_in_months_hours,'Daily energy intensity [${kWh/m^2}$]':Eg_daily_list_Wh_per_meter2,'Average temperature [$^{\circ}$C]':average_temp_list}

df = pd.DataFrame(index = months_list,data = data_atmospheric_conditions)

class ReportComponent(BaseReportComponent):



    @property
    def reported_object(self):

        from .....solvers.linear import ODESystem
        
        if isinstance(self._reported_object, ODESystem):
            return self._reported_object

        else:
            return self._reported_object
            
    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj

    @property
    def _system(self):
        print('Kod do poprawienia #################### bo stosujesz starą zmienną self._system')
        print('zamień se self._system na self.reported_object')
        
        return self.reported_object

#git        
        
class CocalcLoginComponent(ReportComponent):
    
    title="Logowanie do CoCalc"


    def append_elements(self):

        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage


        display(ReportText('''Link do CoCalca: [LINK](https://cocalc.com/) '''))

        pic1 = Picture('./dynpy/utilities/components/guides/images/ColacSignIn.png', caption = "Przed Zalogowaniem",width='9cm')

        display(pic1)

        display(ReportText('Teraz należy zalogować się'))

        pic2 = Picture('./dynpy/utilities/components/guides/images/MetLogow.png', caption = "Metody logowania",width='9cm')

        display(pic2)

        pic3 = Picture('./dynpy/utilities/components/guides/images/Zalogowany.png', caption = "Zalogowany",width='9cm')

        display(pic3)

        display(ReportText('''Poprzez tego linka można dołączyć do projektu Ongoing: [ONGOING](https://cocalc.com/app?project-invite=dS62jsbRJvcfj2Mu) '''))
        


#zostawic to
class CocalcFolderComponent(ReportComponent):
    
    title="Tworzenie Folderu i Jupytera"


    def append_elements(self):
        
        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage

        display(ReportText('Tworzenie Folderu i Jupytera'))
        
        pic4 = Picture('./dynpy/utilities/components/guides/images/OtwarcieOngo.png', width='9cm')
        
        display(pic4)
        
        display(ReportText('W wyszukiwarce szukamy "Ongoing" i otwieramy projekt  '))
        
        pic5 = Picture('./dynpy/utilities/components/guides/images/PoWejsciuWOngo.png', width='9cm')
        
        display(pic5)
        
        display(ReportText('Otworzenie folderu UCZESTNICY  '))
        
        
        display(ReportText('Nadajemy nazwę folderu od naszego imieniem i nazwiskiem jak na zalączonym obrazku i klikamy create folder:'))
        
        pic6 = Picture('./dynpy/utilities/components/guides/images/NazwanieFolderu.png', width='9cm')
        
        display(pic6)
        
        
        display(ReportText('Po utworzeniu folderu klikamy przycisk NEW i utworzymy pierwszego Jupytera. W tym celu po nazwaniu Jupytera wybieramy zakladke Jupyter Notebook.'))
        
        
        pic7 = Picture('./dynpy/utilities/components/guides/images/WyborKernela.png', width='9cm')
        
        display(pic7)
        
        display(ReportText('Wybieramy Python 3 (system - wide)  '))
        display(ReportText('Po wybraniu Kernela utworzony zostanie nasz Jupyter.'))
        
        pic8 = Picture('./dynpy/utilities/components/guides/images/JupekUtworzony.png', width='9cm')
        
        display(pic8)


#podstawy raportowania
        



        
code_dynsys_list_str = '''
#komendy importujące biblioteki
import sympy 
from sympy import Symbol

from dynpy.models.mechanics.pendulum import Pendulum

#komenda wywołująca preview
Pendulum().preview()

#komenda wywołująca pomocne informacje dotyczące wahadła
help(Pendulum)

'''


class CocalcDynSysListComponent(ReportComponent):
    
    title="Lista systemów dynamicznych"


    def append_elements(self):

        system = self.reported_object # it's useless in the case of permanent content - it's commented for future usage
        
        display(ReportText(' Możemy również wyświetlić podgląd takiego wahadła używając poniższej funkcji:  '))
        display(Picture('./dynpy/utilities/components/guides/images/preview.jpg'))
        display(ReportText(' Co więcej, używając poniższej komendy help, możemy otrzymać wszelkie informacje na temat konkretnego modelu:  '))
        display(Picture('./dynpy/utilities/components/guides/images/help.jpg'))

        display(ReportText('''Kod wywołania pomocy jest następujący:'''))
        
        display(GuideCode(   code_dynsys_list_str   ))


        display(ReportText('Poniżej znajduje się lista systemów mechanicznych, których można użyć:  '))
        from dynpy.models import mechanics


        moduly=inspect.getmembers(mechanics, inspect.ismodule)


        for nazwa,modul in moduly:

            mods_tmp = [name for name,cls in inspect.getmembers(modul, inspect.isclass) if issubclass(cls,mechanics.principles.ComposedSystem)]
            classes_str=',\n - '.join(mods_tmp)
            if mods_tmp != []:
                display(ReportText( '\\par' + f'-{nazwa}'+ '\\par'))
                display(ReportText(f'- {classes_str}'))


    
#git
    
class IssuePreparationComponent(pl.IssuePreparationComponent):
    
    title="Issue preparation component"


##########################
##SYNCHRONIZACJA GITHUB### 
##########################

#git

class GitSynchroPanelAccessComponent(ReportComponent):

    title="Dostep do panelu synchronizacji Github"

    @property
    def reported_object(self):


        default_data = {'classname':'ReportingModuleIntroComponent',
                       'module':'guide.en.py',
                       'field':'guide or report',
                       'target':'`ODESystem` class',
                       'issue_no':359,
                       }


        if isinstance(self._reported_object, dict):
            return {**default_data,**self._reported_object}

        elif isinstance(self._reported_object, str):
            return {**default_data,'classname':self._reported_object}

        elif self._reported_object is None:
            return default_data

        else:
            return self._reported_object


    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj
    
    
    def append_elements(self):
#       variables provided by `reported_object` arg
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']

        display(ReportText(f''' Repozytoria Github'a zmieniają się dosyć dynamicznie, a w przypadku, gdy korzysta z niego więcej niż jedna osoba dobrze by było aby każdy miał dostęp do najnowszej wersji. W tym celu przeprowadza się regularnie synchronizację danego rezpozytorium. '''))

        display(ReportText('Jak się dostać do panelu synchronizacji?'))

        display(ReportText(f''' Wybierz projekt, który chcesz zsynchronizować.'''))
        display(Picture('./dynpy/utilities/components/guides/images/1.png', position = 'H', height= NoEscape('11cm'), width = NoEscape('11cm'),caption='Wybór projektu z listy'))

        display(ReportText(f'''Przejdź do foledru, który chcesz zsynchronizować. W naszym przypadku zazwyczaj aktualizujemy moduły.'''))
        display(Picture('./dynpy/utilities/components/guides/images/2.png', position = 'H', height= NoEscape('18cm'), width = NoEscape('18cm'),caption='Wybór folderu do synchronizacji'))

        display(ReportText(f'''Wybierz ikonę VS Code - zostaniesz przeniesiony do nowego okna.'''))
        display(Picture('./dynpy/utilities/components/guides/images/3.png', position = 'H', height= NoEscape('11cm'), width = NoEscape('11cm'),caption='Przycisk przenoszący do VS Code'))

        display(ReportText(f'''Będąc już w panelu VS Code przjedź do zakładki 'Source control' (trzecia od góry), to tu będą przeprowadzane wszystkie operacje.'''))
        display(Picture('./dynpy/utilities/components/guides/images/4.png', position = 'H', height= NoEscape('18cm'), width = NoEscape('18cm'),caption='Source control w panelu VS Code'))

#git

class GitSynchroIntroComponent(GitSynchroPanelAccessComponent):

    title="Wprowadzenie do synchronizacji repozytorium Github"

    def append_elements(self):
        #variables provided by `reported_object` arg
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']
    
        #implement reporting activieties here

        display(ReportText('Synchronizacja repozytorium.'))

        display(ReportText('Przypadek 1. : "Commit" po lewej stronie NIE działa'))

        display(ReportText(f'''Jeśli "Commit" nie działa to kliknij od razu "Sync changes" lub kółeczka synchronizacji na górze'''))
        display(Picture('./dynpy/utilities/components/guides/images/10.png', position = 'H', height= NoEscape('11cm'), width = NoEscape('8cm'),caption='Kółeczko synchronizacji'))
        display(Picture('./dynpy/utilities/components/guides/images/9.png', position = 'H', height= NoEscape('8cm'), width = NoEscape('6cm'),caption='Przycisk, na którym pojawia się "Commit" albo "Sync changes"'))

        display(ReportText('Przypadek 2. : "Commit" po lewej stronie wyświetla się normalnie'))

        display(ReportText(f'''Kliknij "Commit".'''))
        display(Picture('./dynpy/utilities/components/guides/images/5.png', position = 'H', height= NoEscape('8cm'), width = NoEscape('6cm'),caption='Przycisk "Commit"'))

        display(ReportText(f'''Otworzy się opis Commit'a - odkomentuj linijki z modyfikacjami, czyli wszystko od około 4. linii. '''))
        display(Picture('./dynpy/utilities/components/guides/images/6.png', position = 'H', height= NoEscape('13cm'), width = NoEscape('13cm'),caption='''Opis Commit'a '''))

        #display(ObjectCode(example_commit_str))

        display(ReportText(f'''Po odkomentowaniu dopisz tytuł synchronizacji oraz krótkie opisy poszczególnych zmian - jest to ważne w kwestii komunikacji między osobami pracującymi nad projektem. '''))
        display(Picture('./dynpy/utilities/components/guides/images/7.png', position = 'H', height= NoEscape('13cm'), width = NoEscape('11cm'),caption=''' Przykladowy opis Commit'a '''))
        display(ReportText('Przykładowy opisany Commit:'))
        #display(ObjectCode(example_commented_commit_str))
        display(ReportText('Spis przykładowych komenatrzy do Committów:'))
        #display(ObjectCode(example_commit_comments_str))


        display(ReportText(f''' Dla pewności poprawności wprowadzonych zmian warto potwierdzić Commita na chatcie do synchronizacji na Slacku'''))
        display(Picture('./dynpy/utilities/components/guides/images/8_5.png', position = 'H', height= NoEscape('6cm'), width = NoEscape('6cm'),caption='''Kanał do synców '''))

        display(ReportText(f'''Zatwierdź Commit'a klikjąc tick w prawym górnym rogu. Jeśli przycisku nie ma, przełącz karty i spróbuj ponownie.'''))
        display(Picture('./dynpy/utilities/components/guides/images/8.png', position = 'H', height= NoEscape('20cm'), width = NoEscape('20cm'),caption='Zatwierdzanie zmian'))

        display(ReportText(f'''Po zatwierdzeniu, ze strony Github'a takie zmiany wyglądają następująco '''))
        display(Picture('./dynpy/utilities/components/guides/images/7_5.png', position = 'H', height= NoEscape('11cm'), width = NoEscape('8cm'),caption='''Zatwierdzone zmiany z perspektywy Github'a'''))
        display(ReportText('Wystąpienie komunikatu merge'))

        display(ReportText(f'''Taki komunikat będzie oznacza błąd na wyższym szczeblu - nie ruszaj nic więcej i odezwij się do Pana Chilińskiego lub Pana Sierocińskiego. '''))
        display(Picture('./dynpy/utilities/components/guides/images/11.png', position = 'H', height= NoEscape('11cm'), width = NoEscape('8cm'),caption='Error'))
        
       #git
        
class UsageOfGitHubInterfacesComponent(GitSynchroPanelAccessComponent):

    title="Wprowadzenie do GitHubInterface"

    def append_elements(self):
        #variables provided by `reported_object` arg
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']
    
        #implement reporting activieties here
        from dynpy.utilities.creators import GitHubInterface
        display(ReportText('Obsluga rytynowych czynności w dynpy jest wykonanywana przez klasę GitHubInterface'))
        display(ObjectCode(GitHubInterface))
        display(ReportText('Więcej informacji możesz uzyskać w helpie'))
        display(ObjectCode("help(GitHubInterface)"))
        display(ObjectCode(help(GitHubInterface)))


#github

class IssueFeedbackComponent(ReportComponent):

    title="Komentarz zamykajacy issue z komponentami"

    @property
    def reported_object(self):
        from .....dynamics import LagrangesDynamicSystem

        default_data = {'classname':'Component',
                       'module':'dynpy.utilities.components.guides.en',
                       'field':'None',
                       'target':'`ReportComponent` class',
                       'issue_no':359,
                       }

        if isinstance(self._reported_object, dict):
            return {**default_data,**self._reported_object}

        if isinstance(self._reported_object, ReportComponent) or isinstance(self._reported_object, LagrangesDynamicSystem):
            return {**default_data,'classname':self._reported_object.__class__.__name__,'module':self._reported_object.__class__.__module__}
        
        elif isinstance(self._reported_object, str):
            return {**default_data,'classname':self._reported_object}

        elif self._reported_object is None:
            return default_data
        
        else:
            return self._reported_object


    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj


    def append_elements(self):
        #variables provided by `reported_object` arg
        classname = self.reported_object['classname']
        class_module = self.reported_object['module']
        class_field = self.reported_object['field']
        target = self.reported_object['target']

        display(ReportText('The code was checked with the following call: '))
        #display(ObjectCode(git_com_str.format(classname=classname)))
        #display(ObjectCode(git_com_str.format(classname=classname,module=class_module,class_field=class_field)))

class GithubIssueReportComponent(ReportComponent):

    title="Details of GitHub issue"

    def dynamic_title(self):
        
        
        issue = self.reported_object
        if issue==None:
            from github.Issue import Issue
            issue = Issue(requester = 'lsikor', headers = {'title' : 'TEST'}, attributes = {'body':'Przykladowy opis - nie został podany żaden argument', 'title':'TEST', 'number': 999}, completed = False)
        
        return "Details of " + issue.title + "- No: " + str(issue.number)
    
    def append_elements(self):
        
        issue = self.reported_object
        if issue==None:
            from github.Issue import Issue
            issue = Issue(requester = 'lsikor', headers = {'title' : 'TEST'}, attributes = {'body':'Przykladowy opis - nie został podany żaden argument', 'title':'TEST', 'number': 999}, completed = False)
        display(Markdown((f'Issue title: {issue.title} Issue number: {issue.number}')))
        display(ReportText('\\newline'))
        if issue.body is None:
            display(Markdown("No issue description"))
        else:
#             display(Markdown("Issue description: " + issue.body))
            display(IPMarkdown("Issue description: " + issue.body))
        display(ReportText('\\newline'))
        display(ReportText('-'*100))
        display(ReportText('\\newline'))

class UsageOfMeetingCreatorComponent(ReportComponent):
    
    
    title="Github meeting creator class"

    def append_elements(self):
        from .....utilities.creators import MeetingIssueCreator
        display(ReportText('This class is used to create meeting issues through API. The class consists of the following code: \\newline'))
        system = self.reported_object
        display(ObjectCode(MeetingIssueCreator))
        display(ReportText(' \\newline To learn more about this class use help. \\newline'))

git_synchro_intro_call_str=("""
from dynpy.utilities.documents.guides import GithubSynchroGuide
from dynpy.utilities.components.guides.en import *

GithubSynchroGuide();

""")


git_synchro_intro_str =('''Guide consist of bellow components which can be called separatelly:
*GithubSynchroIntroComponent
*GitSynchroPanelAccessComponent
*GitSynchroIntroComponent
*UsageOfGitHubInterfacesComponent
*UsageOfMeetingCreatorComponent
''')

class GithubSynchroIntroComponent(ReportComponent):

    title = "Introduction to basics of GithubSynchronization panel"

    def append_elements(self):

        #from dynpy import mechanics


        display(ReportText('This guide concers basics of GithubSynchronization panel.'))
        #display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(git_synchro_intro_call_str))
        
        display(ReportText(git_synchro_intro_str))


cocalc_setup_str=("""
from dynpy.utilities.documents.guides import IntroToCocalcGuide
from dynpy.utilities.components.guides.en import *

IntroToCocalcGuide(None);

""")

cocalcus_str =('''Guide consist of bellow components which can be called separatelly:
*CocalcUsageComponent
*JupyterSetUpComponent
*CocalcFolderComponent
*CocalcDynSysListComponent
*ReportingBasicsComponent
''')

class CocalcUsageComponent(ReportComponent):

    title = "Introduction to usage of Cocalc"

    def append_elements(self):

        from dynpy.models import mechanics


        display(ReportText('This guide concers basics of Cocalc enviroment capabilities that are crucial for explorating `DynPy` library.'))
#         display(ReportText('Basic call of it is as follows and runs default dynamic system which is `ForcedSpringMassSystem'))
        display(ObjectCode(cocalc_setup_str))
        
        display(ReportText(cocalcus_str))



from sympy import (Symbol, symbols, Matrix, sin, cos, diff, sqrt, S, diag, Eq, Derivative,Integral, Expr,Function,latex, Heaviside, atan, pi)
from numbers import Number
from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex

import numpy as np

from ..dynamics import LagrangesDynamicSystem, HarmonicOscillator, GeometryScene,base_frame,base_origin
from ..utilities.templates import tikz

#from dgeometry import GeometryScene,np

from pylatex import TikZ,TikZNode
import matplotlib.pyplot as plt

import base64
import random
import IPython as IP







class GeometryOfPoint:

    def __init__(self, *args , frame=base_frame, base_origin=base_origin , ivar=Symbol('t')):
        
        #print(type(args),args)
        #name = str(args[0])

        if isinstance(args[0], Point):
            self.Point = args[0]

        elif isinstance(args[0], Expr) or isinstance(args[0], Number):
            P1 = Point('P1')
            P1.set_pos(base_origin, frame.x*args[0] )
            P1.set_vel(frame, frame.x*diff(args[0], ivar) )

            self.Point = P1
            #print(self.Point)

            #print('Expression based on the generalized coordinate')

        elif isinstance(args, tuple):

            if len(args) == 2:
                P1 = Point('P1')
                P1.set_pos(base_origin, frame.x*args[0] + frame.y*args[1])
                P1.set_vel(frame, frame.x*diff(args[0], ivar) + frame.y*diff(args[1], ivar))
                self.Point = P1
                #print(self.Point)

            elif len(args) == 3:
                P2 = Point('P2')
                P2.set_pos(base_origin, frame.x*args[0] + frame.y*args[1] + frame.z*args[2])
                P2.set_vel(frame, frame.x*diff(args[0], ivar) + frame.y*diff(args[1], ivar) + frame.z*diff(args[2], ivar))
                self.Point = P2
                #print(self.Point)

            else:
                print('Tuple out of range')
                P_na = Point('P_na')
                P_na.set_pos(base_origin, frame.x*0)
                P_na.set_vel(frame, frame.x*diff(0, ivar))
                self.Point = P_na

    def get_point(self):
        return self.Point




class Element(LagrangesDynamicSystem):
    """Base class for all elements
    """

    scheme_name = 'damper.PNG'
    real_name = 'damper.PNG'
    #_default_folder_path = "./dynpy/models/images/"
    

    
    # @classmethod
    # def _scheme(cls):

    #     path = cls._default_folder_path + cls.scheme_name

    #     return path

    # @classmethod
    # def _real_example(cls):
    #     path = cls._default_folder_path + cls.real_name

    #     return path
    
    @classmethod
    def _detail_real(cls):
        path = cls._default_folder_path + cls.detail_real_name

        return path
    
    @classmethod
    def _detail_scheme(cls):
        path = cls._default_folder_path + cls.detail_scheme_name

        return path

    def _init_from_components(self, *args, system=None, **kwargs):

        if system is None:
            composed_system = self._elements_sum
        else:
            composed_system = system

        #print('CS',composed_system._components)
        super().__init__(None, system=composed_system)

        #print('self',self._components)
        if self._components is None:
            comps = {}
        else:
            comps = self._components

        self._components = {**comps, **self.components}

    def _all_default_data(self,formatter=None):

        
        return self.get_default_data()
    
    def _all_numerical_data(self):
        
        return self.get_numerical_data()
        
    
    def get_default_data(self):
        return {}

    def get_numerical_data(self):
        return {}
    
    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        if default_data_dict:
            parameters_dict = {
                key: random.choice(items_list)
                for key, items_list in default_data_dict.items()
            }
        else:
            parameters_dict = None

        return parameters_dict

    def get_numerical_parameters(self):

        default_data_dict = self.get_numerical_data()

        if default_data_dict:
            parameters_dict = {
                key: random.choice(items_list)
                for key, items_list in default_data_dict.items()
            }
        else:
            parameters_dict = None

        return parameters_dict
    
    @classmethod
    def preview(cls, example=False):
        if example:
            path = cls._real_example()

        else:
            path = cls._scheme()

        with open(f"{path}", "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read())
        image_file.close()

        return IP.display.Image(base64.b64decode(encoded_string))

    
    @property
    def _components_str(self):

        return None
    

    def __str__(self):
        

        return f'{(self._label)}'    

    def _tikz_2d(self, language='en',*args,**kwargs):


        
        class_name = self.__class__.__name__


        #print(f'{class_name} element check', self.scheme_options )
        return [TikZNode('Material Point',options=['draw'],text=f'{class_name}')]

    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.force)) * Symbol(f'\delta({latex(self._pos1)})')  )
    def _as_force(self):
        #qs=self.qs
        system=self
        eoms_matrix=self.eoms.as_matrix()
        qs_matrix=self.q
        matrix_lenght=len(eoms_matrix)
        force=Force(0,qs_matrix[0],qs_matrix)
        if matrix_lenght!=0:
            for i in range(matrix_lenght):
                force=force+Force(eoms_matrix[i],qs_matrix[i],qs_matrix)
        return force
    def as_force(self):
        return self._as_force()
    
#Pati
class MaterialPoint(Element):
    """
    Model of a Material point with changing point of mass:

    Creates a material point of an inertial body, after inputing correct values of mass -m and general coordinates, which follows a linear motion.
    """
    scheme_name = 'material_point.png'
    real_name = 'material_point.png'
    
    m=Symbol('M',positive=True)
    pos1=Symbol('pos1',positive=True)
    
    z=dynamicsymbols('z')
    
    def __init__(self,
                 m, 
                 pos1 ,
                 qs=None,
                 frame=base_frame,
                 ivar=Symbol('t'),
                 **kwargs):
        if m is not None: self.m=m
        if pos1 is not None: self.pos1=pos1
        self.ivar = ivar
        
        
        if not qs:
            self.qs = [pos1]

        pos1=GeometryOfPoint(pos1).get_point()
            
        if isinstance(pos1, Point):
            
            v=((base_origin.pos_from(pos1).diff(self.ivar,frame).magnitude())**2)
            
            Lagrangian = S.Half * m * ((base_origin.pos_from(pos1).diff(ivar,frame).magnitude())**2)
            #Lagrangian = S.Half * m * ((base_origin.vel(frame).magnitude())**2)
            
        else:
            Lagrangian = S.Half * m * diff(pos1,ivar)**2


        super().__init__(Lagrangian=Lagrangian, qs=qs, ivar=ivar,frame=frame,**kwargs)
        
        self._kinetic_energy = Lagrangian
        
        
    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.m * self.pos1.diff(self.ivar))) * Symbol(f'\delta({latex(self.pos1)})')  )
        
    def symbols_description(self):
        self.sym_desc_dict={
            self.m :r'Mass of body',
            self.pos1 :r'Instantenous position of the body'
        }
        return {**super().symbols_description(),**self.sym_desc_dict}
    
    def get_default_data(self):
        t=self.ivar
        m0=Symbol('m_0',positive=True)
        self.default_data_dict={
            self.m:[m0*no for no in range(1,30)],
        }
        return {**super().get_default_data(),**self.default_data_dict}
    

    def get_numerical_data(self):

        self.default_data_dict={
            self.m:[5* no/100 for no in range(80,120)],
        }
        return self.default_data_dict

    @classmethod
    def from_velocity(cls,
                      m,
                      velocity,
                      qs=None,
                      z=None,
                      ivar=Symbol('t'),
                      frame = base_frame,
                      **kwargs):
        pos1=Integral(velocity,ivar)
        return cls(m=m, pos1=pos1, qs=qs, ivar=ivar, frame = frame)
    @classmethod
    def from_default(cls,
                      m=None,
                      pos1=None,
                      qs=None,
                      ivar=Symbol('t'),
                      frame = base_frame,
                      **kwargs):
        if m is None: m=cls.m
        if pos1 is None: pos1=dynamicsymbols('x')
        if not qs:
            qs=[pos1]
        ivar=cls.ivar
        return cls(m=m, pos1=pos1, qs=qs, ivar=ivar, frame = frame)
    
    def _plot_2d(self, language='en',*args,**kwargs):
      
        class_name =self.__class__.__name__
        
        sch_opt=self.scheme_options
        
        x_shift,y_shift = 0,0
        
        if sch_opt is not None:
            if 'at' in sch_opt:
                x_shift,y_shift = sch_opt['at']
        
        coords = [(0,0),(0,10),(5,10),(5,0),(0,0)]
        x_coords = [  x+x_shift for x,y in  coords ]
        y_coords = [  y+y_shift for x,y in coords]
        
#         my_fun=lambda arg: arg[1]
#         y_coords = list(map(my_fun,  coords))   #[  y for x,y in coords]
#         print(y_coords)
    
        res = GeometryScene.ax_2d.plot(x_coords,y_coords,label=class_name)
        GeometryScene.ax_2d.text(2.5,5,f'${self.m}$',rotation='horizontal',multialignment='center')
        
        

#Mateusz karol
class Spring(Element):
    """
    Model of a Spring: Creates a singular model, after inputing correct values of stiffeness - k and general coordinate(s),
    which analytically display the dynamics of displacing spring after cummulating PE.
    """
    scheme_name = 'spring.PNG'
    real_name = 'spring_real.png'
    
    stiffness=Symbol('k',positive=True)
    l_s=Symbol('l_s', positive=True)
    k0 =Symbol('k_0',positive=True)
    
    def __init__(self, stiffness, pos1, pos2=0, l_s=None ,  qs=None, ivar=Symbol('t'), frame = base_frame,**kwargs):

        if stiffness is not None: self.stiffness=stiffness
        if pos1 is not None: self._pos1=pos1
        self._pos2=pos2
        if l_s is not None: self.l_s=l_s
        else:
            l_s=0
            self.l_s=l_s

        if not qs:
            qs = [pos1]

        Lagrangian = (-S.Half * self.stiffness * ((self.pos2.pos_from(self.pos1)).magnitude() - self.l_s  )**2 )


        super().__init__(Lagrangian=Lagrangian, qs=qs, ivar=ivar, frame=frame,**kwargs)

        self._potential_energy = - Lagrangian
        self._frame = frame

    @property
    def pos1(self):
        return GeometryOfPoint(self._pos1).get_point()

    @property
    def pos2(self):
        return GeometryOfPoint(self._pos2).get_point()

    def force(self):
        if self.pos2==0:
            force = (self.stiffness * self.pos1)
        elif  self.l_s == 0:
            force = self.stiffness * (self.pos2.pos_from(self.pos1)) & self._frame.x
            
        else:
            force = self.stiffness * (self.pos2.pos_from(self.pos1)).magnitude() - self.l_s
        return force
    

    def _plot_2d(self, language='en',*args,**kwargs):

        class_name = self.__class__.__name__

        coords=[(0,0),(0,3),(3,6),(-3,12),(0,15),(0,18)]

        sch_opt=self.scheme_options
        x_shift,y_shift = 0,0
        
        if sch_opt is not None:
            if 'at' in sch_opt:
                x_shift,y_shift = sch_opt['at']

        x_span = np.arange(0,21,1)


        x_coords = np.sin(x_span*np.pi/2)+x_shift
        y_coords = (x_span*0.8)+y_shift
        
        x_end=x_coords[-1]
        y_end=y_coords[-1]

        #print(y_coords)
        
        res = GeometryScene.ax_2d.plot(x_coords,y_coords,label=class_name)
        res =GeometryScene.ax_2d.text(x_end,y_end,'$' + latex(self.stiffness) + '$')

        
        
    def get_default_data(self):

        k0= self.k0
        self.default_data_dict={
            self.stiffness: [S.One *  k0 * no for no in range(80, 135)],
        }
        return {**super().get_default_data(),**self.default_data_dict}
        
    def get_numerical_data(self):

        self.default_numerical_dict={
            self.stiffness: [S.One / 100 *  50e3 * no for no in range(80, 135)],
            
        }
        
        return {**super().get_numerical_data(),**self.default_numerical_dict}
    
    @classmethod 
    def from_default(
            cls,
            stiffness=None,
            pos1=None,
            frame=base_frame,
            qs=None,
            ivar=Symbol('t'),
            **kwargs):
        if stiffness is not None: cls.stiffness=stiffness
        if pos1 is None: pos1= dynamicsymbols('z') 
        if not qs: 
            qs = [pos1] 
        ivar=cls.ivar

        return cls(stiffness=stiffness, pos1=pos1, qs=qs,ivar=ivar, frame = frame)

    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.m * self.pos1.diff(self.ivar))) * Symbol(f'\delta({latex(self.pos1)})')  )
    
   #piotrek 
class TorsionalSpring(Spring):
    """
    Model of a Spring: Creates a singular model, after inputing correct values of stiffeness - k and general coordinate(s),
    which analytically display the dynamics of displacing spring after cummulating PE.
    """
    scheme_name = 'spring.PNG'
    real_name = 'spring_real.png'
    
    stiffness=Symbol('k',positive=True)
    l_s=Symbol('l_s', positive=True)
    k0 =Symbol('k_0',positive=True)
    l0 = Symbol('l_0', positive=True)        
        
    def get_default_data(self):

        k0,l0= self.k0,self.l0
        l=Symbol('l', positive=True)    
        self.default_data_dict={
            #self.stiffness: [S.One *  k0*l0**2 * no for no in range(2,5)],
            self.stiffness: [S.One *  k0*l**2 * no for no in range(2,5)],
        }
        #return {**super().get_default_data(),**self.default_data_dict}
        return {**self.default_data_dict}
        
    def get_numerical_data(self):

        self.default_numerical_dict={
            self.stiffness: [S.One / 100 *  50e3 * no for no in range(80, 135)],
        }
        
        return {**super().get_numerical_data(),**self.default_numerical_dict}    

    


    
    #piotrek
class GravitationalForce(Element):
    """
    Model of a changing centroid for potential energy. Creates a singular model, after inputing correct values of gravity field - g,
    mass of - m as well as additionaly the general coordiante
    """
    scheme_name = ''
    real_name = ''
    def __init__(self, m, g, pos1, qs=None, ivar=Symbol('t')):

        if not qs:
            qs=[pos1]
        self.g = g

        Lagrangian = -(m * g * (pos1))
        


        super().__init__(Lagrangian=Lagrangian, qs=qs, ivar=ivar)

        self._potential_energy = - Lagrangian
        
    def get_numerical_data(self):
        self.default_data_dict={
            self.g: [9.81]
        }
        return self.default_data_dict

    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.m * self.pos1.diff(self.ivar))) * Symbol(f'\delta({latex(self.pos1)})')  )
    
#karol
class Disk(Element):
    """
    Model of a Disk:
    Creates a singular model, after inputing correct values of moment of inertia - I and rotational general coordinate, which analytically displays the dynamics of a rotating wheel.
    """
    scheme_name = 'disk.png'
    real_name = 'disk.png'
    def __init__(self, I, pos1 , qs=None,  frame=base_frame, ivar=Symbol('t')):
        
        if not qs:
            self.qs = [pos1]

        pos1=GeometryOfPoint(pos1).get_point()

        if isinstance(pos1, Point):
            Lagrangian = S.Half * I * ((base_origin.pos_from(pos1).diff(ivar,frame).magnitude())**2)
                
        else:
            Lagrangian = S.Half * I * diff(pos1,ivar)**2


        super().__init__(Lagrangian=Lagrangian, qs=qs, ivar=ivar, frame=frame)
        self._kinetic_energy = Lagrangian

        

class RigidBody2D(Element):
    """
    Model of a 2DoF Rigid body:
    """
    """
    Creates a singular model, after inputing correct values of moment of inertia - I for rotational element and mass - m for linear element and general coordinates, which             analytically display the dynamics of rotating and propeling object: beam.
    """
    scheme_name = 'rigid_body2D.png'
    real_name = 'rigid_body2D.png'
    def __init__(self, m, I, pos_lin=0, pos_rot=0, qs=None, frame=base_frame, ivar=Symbol('t')):
        
        if pos_lin == 0 and pos_rot == 0:
            self.qs = [pos_lin, pos_rot]

        if isinstance(pos_lin, Point) and isinstance(pos_rot, Point):
            Lagrangian = S.Half * m * ((base_origin.pos_from(pos_lin).diff(ivar,frame).magnitude())**2) + S.Half * I *((base_origin.pos_from(pos_rot).diff(ivar,frame).magnitude())**2)
        
        else:
            Lagrangian = S.Half * m * diff(pos_lin,ivar)**2 + S.Half * I * diff(pos_rot,ivar)**2

        
        super().__init__(Lagrangian=Lagrangian, qs=qs, ivar=ivar, frame=frame)
        
        self._kinetic_energy = Lagrangian
        
class Damper(Element):
    """
    Model of a Damper:

    Creates a singular model, after inputing correct values of the damping coefficient - c and general coordinates, which establishes a damping force directly proportional to the velocity, in time, of an inertial virbating element.
    """
    scheme_name = 'damper.PNG'
    real_name = 'damper.PNG'
    
    c= Symbol('c', positive=True)

    def __init__(self, c, pos1, pos2=0, qs=None, ivar=Symbol('t'), frame=base_frame):
        
        if c is not None: self.c=c
        self.pos1=pos1
        self.ivar=ivar
        
        if not qs:
            self.qs = [pos1]

        if isinstance(pos1, Point):
            pos1=GeometryOfPoint(pos1,frame=frame).get_point()
            pos2=GeometryOfPoint(pos2,frame=frame).get_point()
            
            
            
            D = S.Half * c * ((pos1.vel(frame)-pos2.vel(frame)).magnitude()**2).doit()  # pos1.vel(frame).magntidue() behaves as diff(pos1,ivar)
        else:
            D = S.Half * c * (diff(pos1-pos2,ivar))**2

        points_dict={}
        for coord in qs:
            
            coord_vel=diff(coord,ivar)
            
            P_tmp = Point(f'P_{str(coord)}')
            P_tmp.set_vel(frame, coord_vel * frame.x)
            points_dict[coord_vel]=P_tmp

    
        forcelist = [ (point_dmp,-diff(D,coord_vel)*frame.x)  for coord_vel,point_dmp in points_dict.items() ]
        #print(forcelist)
        
        super().__init__(0, qs=qs, forcelist=forcelist, frame=frame, ivar=ivar)
        self._dissipative_potential = D

    @property
    def force(self):
        
        return self.c * diff(self.q[0],self.ivar)
     
    def _plot_2d(self, language='en',*args,**kwargs):

        class_name = self.__class__.__name__

        sch_opt=self.scheme_options
        
        comps = self.components
        
        x_shift,y_shift = 0,0
        
        if sch_opt is not None:
            if 'at' in sch_opt:
                x_shift,y_shift = sch_opt['at']

        res = GeometryScene.ax_2d.plot(np.array([0,0,1,-1]) + x_shift+2,np.array([-1,2,2,2]) +  y_shift, label=class_name, color = 'k') + GeometryScene.ax_2d.plot(np.array([1.5,1.5,0,0,0,-1.5,-1.5]) + x_shift+2,np.array([1,3,3,5,3,3,1]) + y_shift, label=class_name, color = 'k')
        
        text = GeometryScene.ax_2d.text(np.array([1.5]),np.array([3]),f'{class_name}',multialignment='center')
        
    def get_default_data(self):

        t=self.ivar
        c0=Symbol('c_0',positive=True)
        self.default_data_dict={
            self.c:[c0*no for no in range(1,10)],
        }
        return {**super().get_default_data(),**self.default_data_dict}    
    def get_numerical_data(self):

        self.default_numerical_dict={
            #self.stiffness: [S.One / 100 *  50e3 * no for no in range(80, 135)],
            self.c: [S.One   * no for no in range(50, 60)]
        }
        return self.default_numerical_dict

class EngineMount(Spring):

    """
    Model of a Engine Mount: Creates a singular model, after inputing correct values of stiffeness - k and general coordinate(s),
    which analytically display the dynamics of displacing spring after cummulating PE.
    """
    real_path='buick_regal_3800.jpg'#buick_regal_3800.jpg
    scheme_name = 'spring.PNG'
    def pin_diameter(self):
        #średnica sworznia ze wzoru na ścinanie
        kt=Symbol('k_t', positive=True)
        Re=Symbol('R_e', positive=True)

        return sqrt((4*S.One*self.force())/(pi*kt*Re)).doit()

    def pin_force(self):

        return self.force()

    def get_numerical_data(self):
        self.default_data_dict={
            self.stiffness: [1e5 * no for no in range(1,8)]
        }
        return self.default_data_dict
    
    def _plot_2d(self, language='en',*args,**kwargs):

        class_name = self.__class__.__name__

        coords=[(0,0),(0,3),(3,6),(-3,12),(0,15),(0,18)]

        sch_opt=self.scheme_options
        x_shift,y_shift = 0,0
        
        if sch_opt is not None:
            if 'at' in sch_opt:
                x_shift,y_shift = sch_opt['at']

        x_span = np.arange(0,21,1)


        x_coords = np.sin(x_span*np.pi/2)+x_shift
        y_coords = (x_span*0.4)+y_shift
        
        x_end=x_coords[0]+2
        y_end=y_coords[0]/2

        #print(y_coords)
        
        res = GeometryScene.ax_2d.plot(x_coords,y_coords,label=class_name)
        res = GeometryScene.ax_2d.plot(x_end,y_end,'*r')
        res =GeometryScene.ax_2d.text(x_end,y_end,"pin")
        res =GeometryScene.ax_2d.text(x_end,y_end,'$' + latex(self.stiffness) + '$')
    
class DampedEngineMount(EngineMount):

    """
    Model of a Damped Engine Mount: Creates a singular model, after inputing correct values of stiffeness - k, damping coefficient - c and general coordinate(s),
    which analytically display the dynamics of displacing spring after cummulating PE and damper after commulating KE.
    """
    
    real_path='buick_regal_3800.jpg' #buick_regal_3800.jpg
    scheme_name = 'spring.PNG'
    scheme_name = 'damped_engine_vertical_spring_fy.png'
    real_name = 'paccar.jpg'
    detail_scheme_name = 'sruba_pasowana.png'
    detail_real_name = 'buick_regal_3800.jpg'
    
    stiffness = Symbol('k', positive=True)
    damping_coeff = Symbol('c', positive=True)
    _pos1 = dynamicsymbols('z')
    #_pos2 = Symbol('pos_2', positive=True)
    _pos2 = 0
    l_s = Symbol('l_s', positive= True)
    ivar = Symbol('t')
    
    def __init__(
            self,
            stiffness,
            damping_coeff,
            pos1,
            pos2=0,
            l_s=None,
            qs=None,
            ivar=Symbol('t'),
            **kwargs):

        if stiffness is not None: self.stiffness = stiffness
        if damping_coeff is not None:  self.damping_coeff = damping_coeff
        if pos1 is not None: self._pos1=pos1
        if pos2 == 0: self._pos2 = 0
        else: self._pos2 = pos2
        if qs is not None: self.qs=qs
        else: self.qs=[self._pos1]
        if l_s is not None: self.l_s=l_s
        else:
            l_s=0
            self.l_s=l_s
        self.ivar = ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        
        self._elasticity = EngineMount(self.stiffness, pos1=self.pos1, pos2=self.pos2, l_s=self.l_s, qs=self.qs)(label='Elasticity',scheme_options={'at':(2,4)})
        self._damping = Damper(self.damping_coeff, pos1=self.pos1, pos2=self.pos2, qs=self.qs)(label='Damping',scheme_options={'at':(-2,4)})

        components['_elasticity'] = self._elasticity
        components['_damping'] = self._damping

        return components

    def force(self):
        return self._engine_mount.force() + self._damper.force()         #WorkInProgress

    @classmethod 
    def from_default(cls,
                stiffness=None,
                damping_coeff=None,
                pos1=None,
                pos2=0,
                l_s=None,
                qs=None,
                ivar=Symbol('t'),
                 **kwargs):

        return cls(stiffness=stiffness, damping_coeff=damping_coeff, pos1=pos1, pos2=pos2, l_s=l_s, qs=qs,ivar=ivar)

    def _plot_2d(self, language='en',*args,**kwargs):

        class_name = self.__class__.__name__

        coords=[(0,0),(0,3),(3,6),(-3,12),(0,15),(0,18)]

        sch_opt=self.scheme_options

        x_shift,y_shift = 0,0

        if sch_opt is not None:
            if 'at' in sch_opt:
                x_shift,y_shift = sch_opt['at']

        x_span = np.arange(0,21,1)

        x_coords = np.sin(x_span*np.pi/2)+x_shift
        y_coords = (x_span*0.4)+y_shift
        
        x_end=x_coords[0]+2
        y_end=y_coords[0]/2

        #print(y_coords)
#         for comp in self.components.values():
#             comp._plot_2d(language=language, *args, **kwargs)
        res = GeometryScene.ax_2d.plot(x_coords,y_coords,label=class_name)
        res = GeometryScene.ax_2d.plot(x_end,y_end,'*r')
        res =GeometryScene.ax_2d.text(x_end,y_end,"pin")
        res =GeometryScene.ax_2d.text(x_end,y_end,'$' + latex(self.stiffness) + '$')
    
    
    
class Excitation(Element):
    """
    Model of a harmonic extorsion applied onto the elemnt:
    """
    scheme_name = ''
    real_name = ''
    def __init__(self, f, pos_rot, ivar=Symbol('t'), frame=base_frame):
        
        qs = [pos_rot]
        
        pos_rot = GeometryOfPoint(pos_rot).get_point()
        
        dpos_rot = diff(pos_rot,ivar)
        
        P = Point('P')
        P.set_vel(frame, dpos_rot * frame.x)
        P.vel(frame)

        F = f * cos(dpos_rot*ivar) + f * sin(dpos_rot*ivar)
        
        forcelist = [(P, F*frame.x)]
        
        super().__init__(0, qs=qs, forcelist=forcelist, frame=frame, ivar=ivar)
        

class Force(Element):
    """
    Creates enforcement.
    """
    scheme_name = ''
    real_name = ''
    def __init__(self,
                 force,
                 pos1 = None,
                 qs = None,
                 ivar=Symbol('t'),
                 frame = base_frame):

        self._pos1 = pos1
        self.force=force
        if qs is None:
            qs = [pos1]
            
        else:
            qs = qs
    
        if isinstance(pos1, Point):
            P = pos1
            if qs is None:
                diffs=P.vel(frame).magnitude().atoms(Derivative)
                
                qs= [deriv.args[0] for deriv in diffs]
                print(qs)
        else:
            
            P = Point('P')
            P.set_vel(frame,pos1.diff(ivar)*frame.x)
            force=+force*frame.x

        forcelist=[(P,force)]
        
        #display(forcelist)
        
        super().__init__(0, qs=qs, forcelist=forcelist, frame=frame, ivar=ivar)
        
    def get_default_data(self):
        t=self.ivar
        F0=Symbol('F_0',positive=True)
        self.default_data_dict={
            # self.force:[F0*no for no in range(1,10)],
        }
        return {**super().get_default_data(),**self.default_data_dict}
        
    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.force)) * Symbol(f'\delta({latex(self._pos1)})')  )    
        
        
        

        
# class CombustionEngine(Force):

#     eta= Symbol('eta', positive = True)
#     n_engine = Symbol('n')
#     omega=Symbol('omega')

#     def __init__(self, omega, characteristic=None, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):

#         if characteristic is None:
#             self._characteristic= self.characteristic
#         else:
#             self._characteristic =  characteristic
#         if eta is not None: self.eta = eta
#         if omega is not None: self.omega = omega
            

#         super().__init__(self.eta*self._characteristic, omega.integrate(ivar), qs=qs, ivar=ivar)

#     @property
#     def characteristic(self):
#         n_engine = self.omega*60/(2*np.pi)
#         return -4e-5*self.n_engine**2 + 0.2157*self.n_engine + 113.07

#     @classmethod
#     def from_data(cls, data, n, degree=5, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):

#         omega = 2*3.14/60*n
        
#         x = data.index.to_numpy()
#         y = data.iloc[:,0].to_numpy()

#         coeffs=reversed((np.polyfit(x, y, degree)))

        
#         omega_min = 2*3.14/60*x[0]
#         omega_max = 2*3.14/60*x[-1]
        
#         char_slope = 10
        
#         window   =   ((S.One/2+atan(char_slope*(omega-omega_min))/np.pi))    -  ((S.One/2+atan(char_slope*(omega-omega_max))/np.pi))
#         #window = Heaviside(omega-omega_min) - Heaviside(omega-omega_max)
        
#         omega = 2*3.14/60*n
        
#         char_poly  =window *sum([ coeff * n ** no for no, coeff in enumerate(coeffs)])
        

        
#         obj = cls(omega = omega, characteristic = char_poly, eta=eta, qs = qs, ivar = ivar, frame = frame)
#         obj._char_poly  =  char_poly
        
#         return obj

#     @classmethod
#     def from_rpm(cls, n, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):
#         omega = 2*3.14/60*n
#         return cls(omega = omega, eta = eta, qs=qs, ivar=ivar, frame = frame)

class CombustionEngine(Force):

    eta= Symbol('eta', positive = True)
    n_engine = Symbol('n')
    omega=Symbol('omega')
    n_min = Symbol('n_min')
    n_max = Symbol('n_max')

    def __init__(self, omega, characteristic=None, eta = None, n_min=None , n_max=None, qs=None, ivar=Symbol('t'),frame = base_frame):

        if characteristic is None:
            self._characteristic= self.characteristic
        else:
            self._characteristic =  characteristic
        if eta is not None: self.eta = eta
        if omega is not None: self.omega = omega
        if n_min is not None: self.n_min = n_min
        if n_max is not None: self.n_max = n_max

        super().__init__(self.eta*self._characteristic, omega.integrate(ivar), qs=qs, ivar=ivar)

    @property
    def characteristic(self):
        n_engine = self.omega*60/(2*np.pi)
        return -4e-5*self.n_engine**2 + 0.2157*self.n_engine + 113.07

    @classmethod
    def from_data(cls, data, n, degree=5, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):

        omega = 2*3.14/60*n
        
        x = data.index.to_numpy()
        y = data.iloc[:,0].to_numpy()

        coeffs=reversed((np.polyfit(x, y, degree)))

        
        omega_min = 2*3.14/60*x[0]
        omega_max = 2*3.14/60*x[-1]
        
        char_slope = 10e60
        
        window   =   ((S.One/2+atan(char_slope*(omega-omega_min))/np.pi))    -  ((S.One/2+atan(char_slope*(omega-omega_max))/np.pi))
        #window = Heaviside(omega-omega_min) - Heaviside(omega-omega_max)
        
        omega = 2*3.14/60*n
        
        char_poly  =window *sum([ coeff * n ** no for no, coeff in enumerate(coeffs)])
        

        
        obj = cls(omega = omega, characteristic = char_poly, eta=eta, qs = qs, ivar = ivar, frame = frame)
        obj._char_poly  =  char_poly
        
        return obj
    
    @classmethod
    def from_data_gearbox(cls, data, n, degree=5, eta = None, n_min=None , n_max=None , qs=None, ivar=Symbol('t'),frame = base_frame):

        omega = 2*3.14/60*n
        
        x = data.index.to_numpy()
        y = data.iloc[:,0].to_numpy()

        coeffs=reversed((np.polyfit(x, y, degree)))

        if n_min is not None:
            omega_min =  2*3.14/60*n_min
        else:
            omega_min = 2*3.14/60*x[0]
        if n_min is not None:
            omega_max =  2*3.14/60*n_max
        else:
            omega_max = 2*3.14/60*x[0]
        
        char_slope = 10e60
        
        window   =   ((S.One/2+atan(char_slope*(omega-omega_min))/np.pi))    -  ((S.One/2+atan(char_slope*(omega-omega_max))/np.pi))
        #window = Heaviside(omega-omega_min) - Heaviside(omega-omega_max)
        
        omega = 2*3.14/60*n
        
        char_poly  =window *sum([ coeff * n ** no for no, coeff in enumerate(coeffs)])
        
        obj = cls(omega = omega, characteristic = char_poly, eta=eta, qs = qs, ivar = ivar, frame = frame)
        obj._char_poly  =  char_poly
        
        return obj
    
    @classmethod
    def from_data_raw(cls, data, n, degree=5, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):

        omega = 2*3.14/60*n
        
        x = data.index.to_numpy()
        y = data.iloc[:,0].to_numpy()

        coeffs=reversed((np.polyfit(x, y, degree)))

        
        omega_min = 2*3.14/60*x[0]
        omega_max = 2*3.14/60*x[-1]
        
        char_slope = 10e60
        
        window = 1
        #window   =   ((S.One/2+atan(char_slope*(omega-omega_min))/np.pi))    -  ((S.One/2+atan(char_slope*(omega-omega_max))/np.pi))
        #window = Heaviside(omega-omega_min) - Heaviside(omega-omega_max)
        
        omega = 2*3.14/60*n
        
        char_poly  =window *sum([ coeff * n ** no for no, coeff in enumerate(coeffs)])
        

        
        obj = cls(omega = omega, characteristic = char_poly, eta=eta, qs = qs, ivar = ivar, frame = frame)
        obj._char_poly  =  char_poly
        
        return obj
    
    @classmethod
    def from_rpm(cls, n, eta = None, qs=None, ivar=Symbol('t'),frame = base_frame):
        omega = 2*3.14/60*n
        return cls(omega = omega, eta = eta, qs=qs, ivar=ivar, frame = frame)
    
class IntegralElement(Force):
    """
    Model of an Integral Element :
        Creates a additional force which allows to control output value.

     Examples
        =======
        An implementation of the control system's integral element:
        
        Example No.1 - error is defined as the expresion:
        >>> t = symbols('t')
        >>> I,reference = symbols('I reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> error = reference - x
        >>> IntegralElement(I , error = error , target = x , qs=[x])
        
        Example No.2 - direct parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> IntegralElement(I , error = x  , reference = reference , qs=[x])
        
        Example No.3 - indirect parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> vel = diff(x,t)
        >>> IntegralElement(I , error = vel , target = x , reference = reference, qs=[x])

        -We define the symbols and dynamicsymbols
        -We choose regulation strategy
        -We initiate an object IntegralElement in proper way
    
    """
    k_I = Symbol('k_I', positive=True)
    error = dynamicsymbols('e')
    target = dynamicsymbols('target')
    reference = S.Zero

    def __init__(self, k_I, error = None, target = None, reference = None,  qs=None, ivar=Symbol('t'), frame = base_frame):

        if reference is not None: self.reference = reference
        if error is not None: self.error = error

        if target is None and isinstance(self.error,Function):
               target=self.error

        if isinstance(self.error,Expr) and self.reference == S.Zero:
            super().__init__(k_I*self.error.integrate(ivar), target, qs=qs, ivar=ivar)
        else:
            super().__init__(k_I*(self.reference - self.error).integrate(ivar), target, qs=qs, ivar=ivar)


    def _plot_2d(self, language='en'):

        class_name = self.__class__.__name__

        x_coords = np.array([0,0,5,5,0])
        y_coords = np.array([0,3,3,0,0])

        res = GeometryScene.ax_2d.plot(x_coords,y_coords, label=class_name, color = 'k') + GeometryScene.ax_2d.plot([-1,0],[1.5,1.5], label=class_name, color = 'k') + GeometryScene.ax_2d.plot([6,5],[1.5,1.5], label=class_name, color = 'k')
        
        text = GeometryScene.ax_2d.text(np.array([1]),np.array([1.5]),f'{class_name}',multialignment='center')
        
    def _tikz_scheme(self, language='en'):

        class_name = self.__class__.__name__

        schm = tikz.TikzStandalone()
        tk_node = TikZ()
        
        with schm.create(TikZ()) as tkz:
            with tkz.create(TikZNode('Integral Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
            with tkz.create(TikZNode('Integral Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
        return schm
        
class DerivativeElement(Force):
    """
    Model of a Derivative Element :
        Creates an additional force which allows to control output value.

     Examples
        =======
        An implementation of the control system's derivative element:
        
        Example No.1 - error is defined as the expresion:
        >>> t = symbols('t')
        >>> I,reference = symbols('I reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> error = reference - x
        >>> DerivativeElement(I , error = error , target = x , qs=[x])
        
        Example No.2 - direct parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> DerivativeElement(I , error = x  , reference = reference , qs=[x])
        
        Example No.3 - indirect parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> vel = diff(x,t)
        >>> DerivativeElement(I , error = vel , target = x , reference = reference, qs=[x])

        -We define the symbols and dynamicsymbols
        -We choose regulation strategy
        -We initiate an object DerivativeElement in proper way
    
    """
    k_D = Symbol('k_D', positive=True)
    error = dynamicsymbols('e')
    target = dynamicsymbols('target')
    reference = S.Zero
    
    def __init__(self, k_D, error = None, target=None, reference = None,  qs=None, ivar=Symbol('t'), frame = base_frame):
        
        if reference is not None: self.reference = reference
        if error is not None: self.error = error

        if target is None and isinstance(self.error,Function):
               target=self.error

        if isinstance(self.error,Expr) and self.reference == S.Zero:
            super().__init__(k_D*diff(self.error,ivar), target, qs=qs, ivar=ivar)
        else:
            super().__init__(k_D*diff((self.reference - self.error),ivar), target, qs=qs, ivar=ivar)

    def _plot_2d(self, language='en'):

        class_name = self.__class__.__name__

        x_coords = np.array([0,0,5,5,0])
        y_coords = np.array([0,3,3,0,0])

        res = GeometryScene.ax_2d.plot(x_coords,y_coords, label=class_name, color = 'k') + GeometryScene.ax_2d.plot([-1,0],[1.5,1.5], label=class_name, color = 'k') + GeometryScene.ax_2d.plot([6,5],[1.5,1.5], label=class_name, color = 'k')
        
        text = GeometryScene.ax_2d.text(np.array([1.2]),np.array([1.5]),f'{class_name}',multialignment='center')
        
    def _tikz_scheme(self, language='en'):

        class_name = self.__class__.__name__

        schm = tikz.TikzStandalone()
        tk_node = TikZ()
        
        with schm.create(TikZ()) as tkz:
            with tkz.create(TikZNode('Derivative Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
            with tkz.create(TikZNode('Derivative Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
        return schm


class ProportionalElement(Force):
    """
    Model of a Proportional Element :
        Creates an additional force which allows to control output value.

     Examples
        =======
        An implementation of the control system's proportional element:
        
        Example No.1 - error is defined as the expresion:
        >>> t = symbols('t')
        >>> I,reference = symbols('I reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> error = reference - x
        >>> ProportionalElement(I , error = error , target = x , qs=[x])
        
        Example No.2 - direct parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> ProportionalElement(I , error = x  , reference = reference , qs=[x])
        
        Example No.3 - indirect parameter regulation:
        >>> t = symbols('t')
        >>> I, reference= symbols('I, reference')
        >>> x = dynamicsymbols('x') # Generalized Coordinates
        >>> vel = diff(x,t)
        >>> ProportionalElement(I , error = vel , target = x , reference = reference, qs=[x])

        -We define the symbols and dynamicsymbols
        -We choose regulation strategy
        -We initiate an object ProportionalElement in proper way
    
    """
    k_P = Symbol('k_P', positive=True)
    error = Symbol('e', positive=True)
    target = Symbol('target', positive=True)
    reference = S.Zero
    
    def __init__(self, k_P, error = None, target=None, reference = None,  qs=None, ivar=Symbol('t'), frame = base_frame):

        if reference is not None: self.reference = reference
        if error is not None: self.error = error

        if target is None and isinstance(self.error,Function):
               target=self.error

        if isinstance(self.error,Expr) and self.reference == S.Zero:
            super().__init__(k_P*(self.error), target, qs=qs, ivar=ivar)
        else:
            super().__init__(k_P*(self.reference - self.error), target, qs=qs, ivar=ivar)

    def _plot_2d(self, language='en'):

        class_name = self.__class__.__name__

        x_coords = np.array([0,0,5,5,0])
        y_coords = np.array([0,3,3,0,0])

        res = GeometryScene.ax_2d.plot(x_coords,y_coords, label=class_name, color = 'k') + GeometryScene.ax_2d.plot([-1,0],[1.5,1.5], label=class_name, color = 'k') + GeometryScene.ax_2d.plot([6,5],[1.5,1.5], label=class_name, color = 'k')
        
        text = GeometryScene.ax_2d.text(np.array([1]),np.array([1.5]),f'{class_name}',multialignment='center')
        
    def _tikz_scheme(self, language='en'):

        class_name = self.__class__.__name__

        schm = tikz.TikzStandalone()
        tk_node = TikZ()
        
        with schm.create(TikZ()) as tkz:
            with tkz.create(TikZNode('Proportional Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
            with tkz.create(TikZNode('Proportional Element', options=['draw'] , text = f'{class_name}')) as node:
                pass
        return schm

        
class PIController(Force):
    
    k_P = Symbol('k_P', positive=True)
    k_I = Symbol('k_I', positive=True)
    error = Symbol('e', positive=True)
    target = Symbol('target', positive=True)
    reference = S.Zero

    def __init__(self,
                 k_P,
                 k_I,
                 error = None,
                 target=None,
                 reference = None,
                 qs=None,
                 ivar=Symbol('t'),
                 frame = base_frame,
                 **kwargs):

        if k_P is not None: self.k_P = k_P
        if k_I is not None: self.k_I = k_I
        
        if reference is not None: self.reference = reference

        if error is not None: self.error = error

        if target is None and isinstance(self.error,Function):
               target=self.error

        self.ivar = ivar
        self.qs = qs

        self._init_from_components(**kwargs)

    @property
    def components(self):
        
        components = {}
        k_P, k_I = self.k_P, self.k_I
        
        self._proporional_elem = ProportionalElement(k_P, self.error, self.reference, qs = self.qs, ivar = self.ivar)
        self._integral_elem = IntegralElement(k_I, self.error, self.reference, qs = self.qs, ivar = self.ivar)
        
        components['proportional_elem'] = self._proporional_elem
        components['integral_elem'] = self._integral_elem
        
        return components
        
class PID(Element):
    """
    Model of a PID controller:

Creates a model of a PID controller (proportional , integral , derivative) which initates a smoother control over delivered system oscillatory system in form of Lagrange method. Taking into account that a PID regulator is ment for a SISO system the model is built for sDoF system, hence in case of building a mDoF the process will require using two PIDs.
    """
    scheme_name = ''
    real_name = ''
    def __init__(self, kp, ki, kd, pos1, qs=None, ivar=Symbol('t'), frame=base_frame):

        if qs:
            qs = [pos1]

        dpos1 = diff(pos1, ivar)

        In = pos1  # In - error
        u = pos1*ki + dpos1*kp + d2pos1*kd
        G = In - u
        
        forcelist = [(P, G*frame.x)]
        
        pos1=GeometryOfPoint(pos1,frame=frame).get_point()
        
        if isinstance(pos1, Point):
            D = S.Half * c * ((pos1.vel(frame)-pos2.vel(frame)).magnitude()**2).doit()  #pos1*ki + pos1.vel(frame)*kp + d2pos1*kd
        else:
            D = S.Half * c * (diff(pos1-pos2,ivar))**2

        points_dict={}
        for coord in qs:
            
            coord_vel=diff(coord,ivar)
            
            P_tmp = Point(f'P_{str(coord)}')
            P_tmp.set_vel(frame, coord_vel * frame.x)
            points_dict[coord_vel]=P_tmp

        forcelist = [ (point_dmp,-diff(G,coord_vel)*frame.x)  for coord_vel,point_dmp in points_dict.items() ]
        
        super().__init__(0, qs=qs, forcelist=forcelist, frame=frame, ivar=ivar)
        
class ConstrainsReaction(Element):
    """
    Creates enforcement.
    """

    scheme_name = ''
    real_name = ''
    def __init__(self,
                 constrain = None,
                 pos1 = None,
                 qs = None,
                 ivar=Symbol('t'),
                 frame = base_frame):

        self.constrain = constrain
        self._pos1 = pos1
#         self.force=force
        if qs is None:
            qs = [pos1]

        else:
            qs = qs


        if isinstance(pos1, Point):
            P = pos1
            if qs is None:
                diffs=P.vel(frame).magnitude().atoms(Derivative)

                qs= [deriv.args[0] for deriv in diffs]
                print(qs)
        else:

            P = Point('P')
            P.set_vel(frame,pos1.diff(ivar)*frame.x)

            force = 0
            for coord  in qs:

                force=+self.constrain.diff(coord)*frame.x


        forcelist=[(P,force)]

        #display(forcelist)

        super().__init__(0, qs=qs, forcelist=forcelist, frame=frame, ivar=ivar)

    def get_default_data(self):
        t=self.ivar
        F0=Symbol('F_0',positive=True)
        self.default_data_dict={
            # self.force:[F0*no for no in range(1,10)],
        }
        return {**super().get_default_data(),**self.default_data_dict}

    def _get_virtual_work(self):
        return Eq(Symbol('\delta W'), Symbol(latex(self.force)) * Symbol(f'\delta({latex(self._pos1)})')  )
